/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * 你可以在不违反许可协议的情况下使用此文件。
 * 你可以在以下地址获取许可证的副本：
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * 除非适用法律要求或书面同意，软件
 * 根据许可证分发是按“原样”基础进行的，
 * 不附带任何明示或暗示的担保或条件。
 * 请参阅许可证以获取特定语言的权限和
 * 限制。
 */

package net.micode.notes.data;

import android.content.ContentValues;
import android.content.Context;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.util.Log;

import net.micode.notes.data.Notes.DataColumns;
import net.micode.notes.data.Notes.DataConstants;
import net.micode.notes.data.Notes.NoteColumns;

// NotesDatabaseHelper 是一个 SQLiteOpenHelper 类的子类，用于管理笔记应用的数据库
public class NotesDatabaseHelper extends SQLiteOpenHelper {
    // 定义了数据库名称
    private static final String DB_NAME = "note.db";

    // 定义了数据库版本号
    private static final int DB_VERSION = 4;

    // 定义了一个内部接口TABLE，包含了表名常量
    public interface TABLE {
        public static final String NOTE = "note";  // 笔记表名
        public static final String DATA = "data";  // 数据表名
    }

    // 定义了日志标签字符串
    private static final String TAG = "NotesDatabaseHelper";

    // 定义了单例实例
    private static NotesDatabaseHelper mInstance;

    // 定义了创建笔记表的SQL语句
    private static final String CREATE_NOTE_TABLE_SQL =
        "CREATE TABLE " + TABLE.NOTE + "(" +
            NoteColumns.ID + " INTEGER PRIMARY KEY," + // 笔记ID，主键
            NoteColumns.PARENT_ID + " INTEGER NOT NULL DEFAULT 0," + // 父文件夹ID，默认为0
            NoteColumns.ALERTED_DATE + " INTEGER NOT NULL DEFAULT 0," + // 提醒日期，默认为0
            NoteColumns.BG_COLOR_ID + " INTEGER NOT NULL DEFAULT 0," + // 背景颜色ID，默认为0
            NoteColumns.CREATED_DATE + " INTEGER NOT NULL DEFAULT (strftime('%s','now') * 1000)," + // 创建日期，默认为当前时间（毫秒）
            NoteColumns.HAS_ATTACHMENT + " INTEGER NOT NULL DEFAULT 0," + // 是否有附件，默认为0
            NoteColumns.MODIFIED_DATE + " INTEGER NOT NULL DEFAULT (strftime('%s','now') * 1000)," + // 修改日期，默认为当前时间（毫秒）
            NoteColumns.NOTES_COUNT + " INTEGER NOT NULL DEFAULT 0," + // 笔记数量，默认为0
            NoteColumns.SNIPPET + " TEXT NOT NULL DEFAULT ''," + // 笔记摘要，默认为空字符串
            NoteColumns.TYPE + " INTEGER NOT NULL DEFAULT 0," + // 笔记类型，默认为0
            NoteColumns.WIDGET_ID + " INTEGER NOT NULL DEFAULT 0," + // 小部件ID，默认为0
            NoteColumns.WIDGET_TYPE + " INTEGER NOT NULL DEFAULT -1," + // 小部件类型，默认为-1
            NoteColumns.SYNC_ID + " INTEGER NOT NULL DEFAULT 0," + // 同步ID，默认为0
            NoteColumns.LOCAL_MODIFIED + " INTEGER NOT NULL DEFAULT 0," + // 是否本地修改，默认为0
            NoteColumns.ORIGIN_PARENT_ID + " INTEGER NOT NULL DEFAULT 0," + // 原始父文件夹ID，默认为0
            NoteColumns.GTASK_ID + " TEXT NOT NULL DEFAULT ''," + // GTasks ID，默认为空字符串
            NoteColumns.VERSION + " INTEGER NOT NULL DEFAULT 0" + // 版本号，默认为0
        ")";

    // 定义了创建 DATA 表的SQL语句
    private static final String CREATE_DATA_TABLE_SQL =
        "CREATE TABLE " + TABLE.DATA + "(" +
            DataColumns.ID + " INTEGER PRIMARY KEY," + // 数据ID，主键
            DataColumns.MIME_TYPE + " TEXT NOT NULL," + // MIME类型，默认为空字符串
            DataColumns.NOTE_ID + " INTEGER NOT NULL DEFAULT 0," + // 关联的笔记ID，默认为0
            NoteColumns.CREATED_DATE + " INTEGER NOT NULL DEFAULT (strftime('%s','now') * 1000)," + // 创建日期，默认为当前时间（毫秒）
            NoteColumns.MODIFIED_DATE + " INTEGER NOT NULL DEFAULT (strftime('%s','now') * 1000)," + // 修改日期，默认为当前时间（毫秒）
            DataColumns.CONTENT + " TEXT NOT NULL DEFAULT ''," + // 数据内容，默认为空字符串
            DataColumns.DATA1 + " INTEGER," + // 自定义数据1
            DataColumns.DATA2 + " INTEGER," + // 自定义数据2
            DataColumns.DATA3 + " TEXT NOT NULL DEFAULT ''," + // 自定义数据3，默认为空字符串
            DataColumns.DATA4 + " TEXT NOT NULL DEFAULT ''," + // 自定义数据4，默认为空字符串
            DataColumns.DATA5 + " TEXT NOT NULL DEFAULT ''" + // 自定义数据5，默认为空字符串
        ")";

    // 定义了创建数据表中note_id索引的SQL语句
    private static final String CREATE_DATA_NOTE_ID_INDEX_SQL =
        "CREATE INDEX IF NOT EXISTS note_id_index ON " +
        TABLE.DATA + "(" + DataColumns.NOTE_ID + "); "; // 为data表中的note_id列创建索引

    /**
     * 当笔记移动到文件夹时，增加文件夹的笔记数量
     */
    private static final String NOTE_INCREASE_FOLDER_COUNT_ON_UPDATE_TRIGGER =
        "CREATE TRIGGER increase_folder_count_on_update "+
        " AFTER UPDATE OF " + NoteColumns.PARENT_ID + " ON " + TABLE.NOTE +
        " BEGIN " +
        "  UPDATE " + TABLE.NOTE +
        "   SET " + NoteColumns.NOTES_COUNT + "=" + NoteColumns.NOTES_COUNT + " + 1" + // 更新父文件夹的笔记数量加1
        "  WHERE " + NoteColumns.ID + "=new." + NoteColumns.PARENT_ID + ";" + // 父文件夹ID为移动后的文件夹ID
        " END";

    /**
     * 当笔记从文件夹中移除时，减少文件夹的笔记数量
     */
    private static final String NOTE_DECREASE_FOLDER_COUNT_ON_UPDATE_TRIGGER =
        "CREATE TRIGGER decrease_folder_count_on_update " +
        " AFTER UPDATE OF " + NoteColumns.PARENT_ID + " ON " + TABLE.NOTE +
        " BEGIN " +
        "  UPDATE " + TABLE.NOTE +
        "   SET " + NoteColumns.NOTES_COUNT + "=" + NoteColumns.NOTES_COUNT + "-1" + // 更新父文件夹的笔记数量减1
        "  WHERE " + NoteColumns.ID + "=old." + NoteColumns.PARENT_ID + // 父文件夹ID为移动前的文件夹ID
        "  AND " + NoteColumns.NOTES_COUNT + ">0" + ";" + // 仅当笔记数量大于0时更新
        " END";

    /**
     * 当向文件夹中插入新笔记时，增加文件夹的笔记数量
     */
    private static final String NOTE_INCREASE_FOLDER_COUNT_ON_INSERT_TRIGGER =
        "CREATE TRIGGER increase_folder_count_on_insert " +
        " AFTER INSERT ON " + TABLE.NOTE +
        " BEGIN " +
        "  UPDATE " + TABLE.NOTE +
        "   SET " + NoteColumns.NOTES_COUNT + "=" + NoteColumns.NOTES_COUNT + " + 1" + // 更新父文件夹的笔记数量加1
        "  WHERE " + NoteColumns.ID + "=new." + NoteColumns.PARENT_ID + ";" + // 父文件夹ID为新笔记的父文件夹ID
        " END";

    /**
     * 当从文件夹中删除笔记时，减少文件夹的笔记数量
     */
    private static final String NOTE_DECREASE_FOLDER_COUNT_ON_DELETE_TRIGGER =
        "CREATE TRIGGER decrease_folder_count_on_delete " +
        " AFTER DELETE ON " + TABLE.NOTE +
        " BEGIN " +
        "  UPDATE " + TABLE.NOTE +
        "   SET " + NoteColumns.NOTES_COUNT + "=" + NoteColumns.NOTES_COUNT + "-1" + // 更新父文件夹的笔记数量减1
        "  WHERE " + NoteColumns.ID + "=old." + NoteColumns.PARENT_ID + // 父文件夹ID为删除笔记的父文件夹ID
        "  AND " + NoteColumns.NOTES_COUNT + ">0;" + // 仅当笔记数量大于0时更新
        " END";

    /**
     * 当插入类型为 {@link DataConstants#NOTE} 的数据时，更新笔记的内容
     */
    private static final String DATA_UPDATE_NOTE_CONTENT_ON_INSERT_TRIGGER =
        "CREATE TRIGGER update_note_content_on_insert " +
        " AFTER INSERT ON " + TABLE.DATA +
        " WHEN new." + DataColumns.MIME_TYPE + "='" + DataConstants.NOTE + "'" + // 当插入的数据类型为笔记时
        " BEGIN" +
        "  UPDATE " + TABLE.NOTE +
        "   SET " + NoteColumns.SNIPPET + "=new." + DataColumns.CONTENT + // 更新笔记的摘要为新插入的数据内容
        "  WHERE " + NoteColumns.ID + "=new." + DataColumns.NOTE_ID + ";" + // 笔记ID为新插入数据关联的笔记ID
        " END";

    /**
     * 当更新类型为 {@link DataConstants#NOTE} 的数据时，更新笔记的内容
     */
    private static final String DATA_UPDATE_NOTE_CONTENT_ON_UPDATE_TRIGGER =
        "CREATE TRIGGER update_note_content_on_update " +
        " AFTER UPDATE ON " + TABLE.DATA +
        " WHEN old." + DataColumns.MIME_TYPE + "='" + DataConstants.NOTE + "'" + // 当更新的数据类型为笔记时
        " BEGIN" +
        "  UPDATE " + TABLE.NOTE +
        "   SET " + NoteColumns.SNIPPET + "=new." + DataColumns.CONTENT + // 更新笔记的摘要为更新后的数据内容
        "  WHERE " + NoteColumns.ID + "=new." + DataColumns.NOTE_ID + ";" + // 笔记ID为更新数据关联的笔记ID
        " END";

    /**
     * 当删除类型为 {@link DataConstants#NOTE} 的数据时，清空笔记的内容
     */
    private static final String DATA_UPDATE_NOTE_CONTENT_ON_DELETE_TRIGGER =
        "CREATE TRIGGER update_note_content_on_delete " +
        " AFTER delete ON " + TABLE.DATA +
        " WHEN old." + DataColumns.MIME_TYPE + "='" + DataConstants.NOTE + "'" + // 当删除的数据类型为笔记时
        " BEGIN" +
        "  UPDATE " + TABLE.NOTE +
        "   SET " + NoteColumns.SNIPPET + "=''" + // 清空笔记的摘要
        "  WHERE " + NoteColumns.ID + "=old." + DataColumns.NOTE_ID + ";" + // 笔记ID为删除数据关联的笔记ID
        " END";

    /**
     * 当删除笔记时，删除属于该笔记的所有数据
     */
    private static final String NOTE_DELETE_DATA_ON_DELETE_TRIGGER =
        "CREATE TRIGGER delete_data_on_delete " +
        " AFTER DELETE ON " + TABLE.NOTE +
        " BEGIN" +
        "  DELETE FROM " + TABLE.DATA + // 删除data表中与笔记ID关联的所有数据
        "   WHERE " + DataColumns.NOTE_ID + "=old." + NoteColumns.ID + ";" + // 笔记ID为删除笔记的ID
        " END";

    /**
     * 当删除文件夹时，删除属于该文件夹的所有笔记
     */
    private static final String FOLDER_DELETE_NOTES_ON_DELETE_TRIGGER =
        "CREATE TRIGGER folder_delete_notes_on_delete " +
        " AFTER DELETE ON " + TABLE.NOTE +
        " BEGIN" +
        "  DELETE FROM " + TABLE.NOTE + // 删除note表中与文件夹ID关联的所有笔记
        "   WHERE " + NoteColumns.PARENT_ID + "=old." + NoteColumns.ID + ";" + // 父文件夹ID为删除文件夹的ID
        " END";

    /**
     * 当文件夹被移动到回收站文件夹时，将文件夹中的所有笔记也移动到回收站文件夹
     */
    private static final String FOLDER_MOVE_NOTES_ON_TRASH_TRIGGER =
        "CREATE TRIGGER folder_move_notes_on_trash " +
        " AFTER UPDATE ON " + TABLE.NOTE +
        " WHEN new." + NoteColumns.PARENT_ID + "=" + Notes.ID_TRASH_FOLER + // 当笔记被移动到回收站文件夹时
        " BEGIN" +
        "  UPDATE " + TABLE.NOTE +
        "   SET " + NoteColumns.PARENT_ID + "=" + Notes.ID_TRASH_FOLER + // 将笔记的父文件夹ID设置为回收站文件夹ID
        "  WHERE " + NoteColumns.PARENT_ID + "=old." + NoteColumns.ID + ";" + // 笔记的父文件夹ID为移动前的文件夹ID
        " END";

    // 构造函数，初始化数据库帮助器
    public NotesDatabaseHelper(Context context) {
        super(context, DB_NAME, null, DB_VERSION);
    }

    // 创建笔记表的方法
    public void createNoteTable(SQLiteDatabase db) {
        db.execSQL(CREATE_NOTE_TABLE_SQL); // 执行创建笔记表的SQL语句
        reCreateNoteTableTriggers(db); // 重新创建笔记表的触发器
        createSystemFolder(db); // 创建系统文件夹
        Log.d(TAG, "note table has been created"); // 记录日志，表示笔记表已创建
    }

    // 重新创建笔记表触发器的方法
    private void reCreateNoteTableTriggers(SQLiteDatabase db) {
        // 删除旧的触发器
        db.execSQL("DROP TRIGGER IF EXISTS increase_folder_count_on_update");
        db.execSQL("DROP TRIGGER IF EXISTS decrease_folder_count_on_update");
        db.execSQL("DROP TRIGGER IF EXISTS decrease_folder_count_on_delete");
        db.execSQL("DROP TRIGGER IF EXISTS delete_data_on_delete");
        db.execSQL("DROP TRIGGER IF EXISTS increase_folder_count_on_insert");
        db.execSQL("DROP TRIGGER IF EXISTS folder_delete_notes_on_delete");
        db.execSQL("DROP TRIGGER IF EXISTS folder_move_notes_on_trash");

        // 创建新的触发器
        db.execSQL(NOTE_INCREASE_FOLDER_COUNT_ON_UPDATE_TRIGGER);
        db.execSQL(NOTE_DECREASE_FOLDER_COUNT_ON_UPDATE_TRIGGER);
        db.execSQL(NOTE_DECREASE_FOLDER_COUNT_ON_DELETE_TRIGGER);
        db.execSQL(NOTE_DELETE_DATA_ON_DELETE_TRIGGER);
        db.execSQL(NOTE_INCREASE_FOLDER_COUNT_ON_INSERT_TRIGGER);
        db.execSQL(FOLDER_DELETE_NOTES_ON_DELETE_TRIGGER);
        db.execSQL(FOLDER_MOVE_NOTES_ON_TRASH_TRIGGER);
    }

    // 创建系统文件夹的方法
    private void createSystemFolder(SQLiteDatabase db) {
        ContentValues values = new ContentValues();

        /**
         * 创建通话记录文件夹，用于存储通话笔记
         */
        values.put(NoteColumns.ID, Notes.ID_CALL_RECORD_FOLDER); // 设置系统文件夹ID
        values.put(NoteColumns.TYPE, Notes.TYPE_SYSTEM); // 设置文件夹类型为系统类型
        db.insert(TABLE.NOTE, null, values); // 插入通话记录文件夹到笔记表

        /**
         * 创建根文件夹，默认文件夹
         */
        values.clear(); // 清空之前插入的值
        values.put(NoteColumns.ID, Notes.ID_ROOT_FOLDER); // 设置系统文件夹ID
        values.put(NoteColumns.TYPE, Notes.TYPE_SYSTEM); // 设置文件夹类型为系统类型
        db.insert(TABLE.NOTE, null, values); // 插入根文件夹到笔记表

        /**
         * 创建临时文件夹，用于移动笔记
         */
        values.clear(); // 清空之前插入的值
        values.put(NoteColumns.ID, Notes.ID_TEMPARAY_FOLDER); // 设置系统文件夹ID
        values.put(NoteColumns.TYPE, Notes.TYPE_SYSTEM); // 设置文件夹类型为系统类型
        db.insert(TABLE.NOTE, null, values); // 插入临时文件夹到笔记表

        /**
         * 创建回收站文件夹
         */
        values.clear(); // 清空之前插入的值
        values.put(NoteColumns.ID, Notes.ID_TRASH_FOLER); // 设置系统文件夹ID
        values.put(NoteColumns.TYPE, Notes.TYPE_SYSTEM); // 设置文件夹类型为系统类型
        db.insert(TABLE.NOTE, null, values); // 插入回收站文件夹到笔记表
    }

    // 创建数据表的方法
    public void createDataTable(SQLiteDatabase db) {
        db.execSQL(CREATE_DATA_TABLE_SQL); // 执行创建数据表的SQL语句
        reCreateDataTableTriggers(db); // 重新创建数据表的触发器
        db.execSQL(CREATE_DATA_NOTE_ID_INDEX_SQL); // 创建note_id索引
        Log.d(TAG, "data table has been created"); // 记录日志，表示数据表已创建
    }

    // 重新创建数据表触发器的方法
    private void reCreateDataTableTriggers(SQLiteDatabase db) {
        // 删除旧的触发器
        db.execSQL("DROP TRIGGER IF EXISTS update_note_content_on_insert");
        db.execSQL("DROP TRIGGER IF EXISTS update_note_content_on_update");
        db.execSQL("DROP TRIGGER IF EXISTS update_note_content_on_delete");

        // 创建新的触发器
        db.execSQL(DATA_UPDATE_NOTE_CONTENT_ON_INSERT_TRIGGER);
        db.execSQL(DATA_UPDATE_NOTE_CONTENT_ON_UPDATE_TRIGGER);
        db.execSQL(DATA_UPDATE_NOTE_CONTENT_ON_DELETE_TRIGGER);
    }

    // 获取 NotesDatabaseHelper 单例实例的方法
    static synchronized NotesDatabaseHelper getInstance(Context context) {
        if (mInstance == null) {
            mInstance = new NotesDatabaseHelper(context); // 初始化单例实例
        }
        return mInstance; // 返回单例实例
    }

    // 当数据库第一次创建时调用的方法
    @Override
    public void onCreate(SQLiteDatabase db) {
        createNoteTable(db); // 创建笔记表
        createDataTable(db); // 创建数据表
    }

    // 当数据库需要升级时调用的方法
    @Override
    public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
        boolean reCreateTriggers = false;
        boolean skipV2 = false;

        // 如果当前版本是1，直接删除旧表并创建新表
        if (oldVersion == 1) {
            upgradeToV2(db); // 升级到版本2
            skipV2 = true; // 这次升级包含了从v2到v3的升级
            oldVersion++;
        }

        // 如果当前版本是2，添加新的列并创建系统文件夹
        if (oldVersion == 2 && !skipV2) {
            upgradeToV3(db); // 升级到版本3
            reCreateTriggers = true; // 标记需要重新创建触发器
            oldVersion++;
        }

        // 如果当前版本是3，添加版本号列
        if (oldVersion == 3) {
            upgradeToV4(db); // 升级到版本4
            oldVersion++;
        }

        // 如果需要重新创建触发器，调用相应的方法
        if (reCreateTriggers) {
            reCreateNoteTableTriggers(db); // 重新创建笔记表的触发器
            reCreateDataTableTriggers(db); // 重新创建数据表的触发器
        }

        // 如果升级失败，抛出异常
        if (oldVersion != newVersion) {
            throw new IllegalStateException("Upgrade notes database to version " + newVersion
                    + " fails"); // 抛出异常，表示数据库升级失败
        }
    }

    // 升级到数据库版本2的方法
    private void upgradeToV2(SQLiteDatabase db) {
        db.execSQL("DROP TABLE IF EXISTS " + TABLE.NOTE); // 删除旧的笔记表
        db.execSQL("DROP TABLE IF EXISTS " + TABLE.DATA); // 删除旧的数据表
        createNoteTable(db); // 创建新的笔记表
        createDataTable(db); // 创建新的数据表
    }

    // 升级到数据库版本3的方法
    private void upgradeToV3(SQLiteDatabase db) {
        // 删除不再使用的触发器
        db.execSQL("DROP TRIGGER IF EXISTS update_note_modified_date_on_insert");
        db.execSQL("DROP TRIGGER IF EXISTS update_note_modified_date_on_delete");
        db.execSQL("DROP TRIGGER IF EXISTS update_note_modified_date_on_update");

        // 添加 gtask id 列
        db.execSQL("ALTER TABLE " + TABLE.NOTE + " ADD COLUMN " + NoteColumns.GTASK_ID
                + " TEXT NOT NULL DEFAULT ''"); // 为笔记表添加gtask_id列

        // 添加回收站系统文件夹
        ContentValues values = new ContentValues();
        values.put(NoteColumns.ID, Notes.ID_TRASH_FOLER); // 设置系统文件夹ID
        values.put(NoteColumns.TYPE, Notes.TYPE_SYSTEM); // 设置文件夹类型为系统类型
        db.insert(TABLE.NOTE, null, values); // 插入回收站文件夹到笔记表
    }

    // 升级到数据库版本4的方法
    private void upgradeToV4(SQLiteDatabase db) {
        // 添加版本号列
        db.execSQL("ALTER TABLE " + TABLE.NOTE + " ADD COLUMN " + NoteColumns.VERSION
                + " INTEGER NOT NULL DEFAULT 0"); // 为笔记表添加version列
    }
}
