/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.gtask.remote; // 定义包名

import android.app.Notification; // 导入用于创建通知的类
import android.app.NotificationManager; // 导入用于管理通知的类
import android.app.PendingIntent; // 导入用于创建待处理Intent的类
import android.content.Context; // 导入上下文类
import android.content.Intent; // 导入Intent类
import android.os.AsyncTask; // 导入异步任务类

import net.micode.notes.R; // 导入资源类
import net.micode.notes.ui.NotesListActivity; // 导入笔记列表活动类
import net.micode.notes.ui.NotesPreferenceActivity; // 导入笔记偏好设置活动类

// GTask 异步任务类，继承自 AsyncTask 类
public class GTaskASyncTask extends AsyncTask<Void, String, Integer> {

    // 定义通知的唯一 ID
    private static int GTASK_SYNC_NOTIFICATION_ID = 5234235;

    // 定义完成监听器接口
    public interface OnCompleteListener {
        void onComplete(); // 完成回调方法
    }

    // 上下文对象
    private Context mContext;

    // 通知管理器对象
    private NotificationManager mNotifiManager;

    // GTask 任务管理器对象
    private GTaskManager mTaskManager;

    // 完成监听器
    private OnCompleteListener mOnCompleteListener;

    // 构造函数
    public GTaskASyncTask(Context context, OnCompleteListener listener) {
        mContext = context; // 初始化上下文
        mOnCompleteListener = listener; // 初始化完成监听器
        mNotifiManager = (NotificationManager) mContext.getSystemService(Context.NOTIFICATION_SERVICE); // 获取通知管理器
        mTaskManager = GTaskManager.getInstance(); // 获取 GTask 任务管理器实例
    }

    // 取消同步方法
    public void cancelSync() {
        mTaskManager.cancelSync(); // 调用任务管理器的取消同步方法
    }

    // 发布进度信息
    public void publishProgess(String message) {
        publishProgress(new String[] {
            message // 创建消息数组并发布进度
        });
    }

    // 显示通知
    private void showNotification(int tickerId, String content) {
        // 创建通知对象
        Notification notification = new Notification(R.drawable.notification, mContext.getString(tickerId), System.currentTimeMillis());
        notification.defaults = Notification.DEFAULT_LIGHTS; // 设置通知的默认灯光
        notification.flags = Notification.FLAG_AUTO_CANCEL; // 设置通知自动取消的标志
        
        // 创建待处理Intent
        PendingIntent pendingIntent;
        // 根据不同的 tickerId 创建不同的 Intent
        if (tickerId != R.string.ticker_success) {
            pendingIntent = PendingIntent.getActivity(mContext, 0, new Intent(mContext, NotesPreferenceActivity.class), 0);
        } else {
            pendingIntent = PendingIntent.getActivity(mContext, 0, new Intent(mContext, NotesListActivity.class), 0);
        }
        
        // 设置通知的内容信息
        notification.setLatestEventInfo(mContext, mContext.getString(R.string.app_name), content, pendingIntent);
        // 显示通知
        mNotifiManager.notify(GTASK_SYNC_NOTIFICATION_ID, notification);
    }

    // 后台执行任务
    @Override
    protected Integer doInBackground(Void... unused) {
        publishProgess(mContext.getString(R.string.sync_progress_login, NotesPreferenceActivity.getSyncAccountName(mContext))); // 发布登录进度
        return mTaskManager.sync(mContext, this); // 调用任务管理器的同步方法并返回结果
    }

    // 更新进度
    @Override
    protected void onProgressUpdate(String... progress) {
        showNotification(R.string.ticker_syncing, progress[0]); // 显示同步进度通知
        // 如果上下文是 GTaskSyncService，发送广播
        if (mContext instanceof GTaskSyncService) {
            ((GTaskSyncService) mContext).sendBroadcast(progress[0]);
        }
    }

    // 任务执行完成后的回调
    @Override
    protected void onPostExecute(Integer result) {
        // 根据结果显示相应的通知
        if (result == GTaskManager.STATE_SUCCESS) {
            showNotification(R.string.ticker_success, mContext.getString(R.string.success_sync_account, mTaskManager.getSyncAccount())); // 显示成功通知
            NotesPreferenceActivity.setLastSyncTime(mContext, System.currentTimeMillis()); // 设置最后同步时间
        } else if (result == GTaskManager.STATE_NETWORK_ERROR) {
            showNotification(R.string.ticker_fail, mContext.getString(R.string.error_sync_network)); // 显示网络错误通知
        } else if (result == GTaskManager.STATE_INTERNAL_ERROR) {
            showNotification(R.string.ticker_fail, mContext.getString(R.string.error_sync_internal)); // 显示内部错误通知
        } else if (result == GTaskManager.STATE_SYNC_CANCELLED) {
            showNotification(R.string.ticker_cancel, mContext.getString(R.string.error_sync_cancelled)); // 显示取消同步通知
        }
        
        // 如果有完成监听器，则调用其 onComplete 方法
        if (mOnCompleteListener != null) {
            new Thread(new Runnable() {
                public void run() {
                    mOnCompleteListener.onComplete(); // 执行完成回调
                }
            }).start(); // 启动新线程执行回调
        }
    }
}
