/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * 该文件遵循 Apache License 2.0
 */

package net.micode.notes.gtask.remote; // 定义包名

import android.app.Activity; // 导入 Android Activity 类
import android.content.ContentResolver; // 导入 ContentResolver 类
import android.content.ContentUris; // 导入 ContentUris 类
import android.content.ContentValues; // 导入 ContentValues 类
import android.content.Context; // 导入 Context 类
import android.database.Cursor; // 导入 Cursor 类
import android.util.Log; // 导入日志工具类

import net.micode.notes.R; // 导入资源类
import net.micode.notes.data.Notes; // 导入笔记数据类
import net.micode.notes.data.Notes.DataColumns; // 导入数据列类
import net.micode.notes.data.Notes.NoteColumns; // 导入笔记列类
import net.micode.notes.gtask.data.MetaData; // 导入元数据类
import net.micode.notes.gtask.data.Node; // 导入节点类
import net.micode.notes.gtask.data.SqlNote; // 导入 SQL 笔记类
import net.micode.notes.gtask.data.Task; // 导入任务类
import net.micode.notes.gtask.data.TaskList; // 导入任务列表类
import net.micode.notes.gtask.exception.ActionFailureException; // 导入操作失败异常类
import net.micode.notes.gtask.exception.NetworkFailureException; // 导入网络失败异常类
import net.micode.notes.tool.DataUtils; // 导入数据工具类
import net.micode.notes.tool.GTaskStringUtils; // 导入 Google 任务字符串工具类

import org.json.JSONArray; // 导入 JSON 数组类
import org.json.JSONException; // 导入 JSON 异常类
import org.json.JSONObject; // 导入 JSON 对象类

import java.util.HashMap; // 导入哈希表类
import java.util.HashSet; // 导入哈希集合类
import java.util.Iterator; // 导入迭代器类
import java.util.Map; // 导入映射类

// GTaskManager 类负责管理 Google 任务的同步操作
public class GTaskManager {
    private static final String TAG = GTaskManager.class.getSimpleName(); // 定义日志标签

    // 定义同步状态常量
    public static final int STATE_SUCCESS = 0; // 成功状态
    public static final int STATE_NETWORK_ERROR = 1; // 网络错误状态
    public static final int STATE_INTERNAL_ERROR = 2; // 内部错误状态
    public static final int STATE_SYNC_IN_PROGRESS = 3; // 同步进行中状态
    public static final int STATE_SYNC_CANCELLED = 4; // 同步取消状态

    private static GTaskManager mInstance = null; // 单例实例

    private Activity mActivity; // 当前 Activity
    private Context mContext; // 上下文对象
    private ContentResolver mContentResolver; // 内容解析器
    private boolean mSyncing; // 是否正在同步
    private boolean mCancelled; // 是否被取消
    private HashMap<String, TaskList> mGTaskListHashMap; // 任务列表哈希表
    private HashMap<String, Node> mGTaskHashMap; // 任务哈希表
    private HashMap<String, MetaData> mMetaHashMap; // 元数据哈希表
    private TaskList mMetaList; // 元任务列表
    private HashSet<Long> mLocalDeleteIdMap; // 本地删除 ID 集合
    private HashMap<String, Long> mGidToNid; // GID 到 NID 映射
    private HashMap<Long, String> mNidToGid; // NID 到 GID 映射

    // 构造函数
    private GTaskManager() {
        mSyncing = false; // 初始化同步状态
        mCancelled = false; // 初始化取消状态
        mGTaskListHashMap = new HashMap<String, TaskList>(); // 初始化任务列表哈希表
        mGTaskHashMap = new HashMap<String, Node>(); // 初始化任务哈希表
        mMetaHashMap = new HashMap<String, MetaData>(); // 初始化元数据哈希表
        mMetaList = null; // 初始化元任务列表
        mLocalDeleteIdMap = new HashSet<Long>(); // 初始化本地删除 ID 集合
        mGidToNid = new HashMap<String, Long>(); // 初始化 GID 到 NID 映射
        mNidToGid = new HashMap<Long, String>(); // 初始化 NID 到 GID 映射
    }

    // 获取单例实例
    public static synchronized GTaskManager getInstance() {
        if (mInstance == null) {
            mInstance = new GTaskManager(); // 创建新实例
        }
        return mInstance; // 返回实例
    }

    // 设置 Activity 上下文，主要用于获取认证令牌
    public synchronized void setActivityContext(Activity activity) {
        mActivity = activity; // 存储当前 Activity
    }

    // 同步方法，执行实际的同步操作
    public int sync(Context context, GTaskASyncTask asyncTask) {
        if (mSyncing) { // 如果已经在同步则返回
            Log.d(TAG, "Sync is in progress");
            return STATE_SYNC_IN_PROGRESS;
        }
        mContext = context; // 设置上下文
        mContentResolver = mContext.getContentResolver(); // 获取内容解析器
        mSyncing = true; // 设置为正在同步
        mCancelled = false; // 重置取消标志
        // 清空各种哈希表和集合
        mGTaskListHashMap.clear();
        mGTaskHashMap.clear();
        mMetaHashMap.clear();
        mLocalDeleteIdMap.clear();
        mGidToNid.clear();
        mNidToGid.clear();

        try {
            GTaskClient client = GTaskClient.getInstance(); // 获取 Google 任务客户端
            client.resetUpdateArray(); // 重置更新数组

            // 登录 Google 任务
            if (!mCancelled) {
                if (!client.login(mActivity)) { // 如果登录失败
                    throw new NetworkFailureException("login google task failed"); // 抛出网络异常
                }
            }

            // 从 Google 获取任务列表
            asyncTask.publishProgess(mContext.getString(R.string.sync_progress_init_list)); // 发布进度信息
            initGTaskList(); // 初始化任务列表

            // 进行内容同步
            asyncTask.publishProgess(mContext.getString(R.string.sync_progress_syncing)); // 发布同步进度
            syncContent(); // 执行内容同步
        } catch (NetworkFailureException e) { // 捕获网络异常
            Log.e(TAG, e.toString());
            return STATE_NETWORK_ERROR; // 返回网络错误状态
        } catch (ActionFailureException e) { // 捕获操作失败异常
            Log.e(TAG, e.toString());
            return STATE_INTERNAL_ERROR; // 返回内部错误状态
        } catch (Exception e) { // 捕获所有其他异常
            Log.e(TAG, e.toString());
            e.printStackTrace(); // 打印堆栈跟踪
            return STATE_INTERNAL_ERROR; // 返回内部错误状态
        } finally { // 最终块，无论如何都会执行
            // 清空各种哈希表和集合
            mGTaskListHashMap.clear();
            mGTaskHashMap.clear();
            mMetaHashMap.clear();
            mLocalDeleteIdMap.clear();
            mGidToNid.clear();
            mNidToGid.clear();
            mSyncing = false; // 重置同步状态
        }

        return mCancelled ? STATE_SYNC_CANCELLED : STATE_SUCCESS; // 返回同步结果
    }

    // 初始化任务列表
    private void initGTaskList() throws NetworkFailureException {
        if (mCancelled) // 如果已取消则返回
            return;
        GTaskClient client = GTaskClient.getInstance(); // 获取 Google 任务客户端
        try {
            JSONArray jsTaskLists = client.getTaskLists(); // 获取任务列表的 JSON 数据

            // 初始化元任务列表
            mMetaList = null; // 初始化元任务列表为 null
            for (int i = 0; i < jsTaskLists.length(); i++) { // 遍历任务列表
                JSONObject object = jsTaskLists.getJSONObject(i); // 获取每个任务列表的 JSON 对象
                String gid = object.getString(GTaskStringUtils.GTASK_JSON_ID); // 获取任务的 GID
                String name = object.getString(GTaskStringUtils.GTASK_JSON_NAME); // 获取任务的名称

                // 判断任务名称是否为元数据文件夹
                if (name.equals(GTaskStringUtils.MIUI_FOLDER_PREFFIX + GTaskStringUtils.FOLDER_META)) {
                    mMetaList = new TaskList(); // 创建新的任务列表
                    mMetaList.setContentByRemoteJSON(object); // 设置任务内容

                    // 加载元数据
                    JSONArray jsMetas = client.getTaskList(gid); // 获取元数据列表
                    for (int j = 0; j < jsMetas.length(); j++) { // 遍历元数据
                        object = (JSONObject) jsMetas.getJSONObject(j); // 获取每个元数据的 JSON 对象
                        MetaData metaData = new MetaData(); // 创建元数据对象
                        metaData.setContentByRemoteJSON(object); // 设置元数据内容
                        if (metaData.isWorthSaving()) { // 如果元数据值得保存
                            mMetaList.addChildTask(metaData); // 将元数据添加到元任务列表中
                            if (metaData.getGid() != null) { // 如果 GID 不为 null
                                mMetaHashMap.put(metaData.getRelatedGid(), metaData); // 将元数据存入哈希表
                            }
                        }
                    }
                }
            }

            // 如果不存在元任务列表，则创建一个新列表
            if (mMetaList == null) {
                mMetaList = new TaskList(); // 创建新的元任务列表
                mMetaList.setName(GTaskStringUtils.MIUI_FOLDER_PREFFIX + GTaskStringUtils.FOLDER_META); // 设置名称
                GTaskClient.getInstance().createTaskList(mMetaList); // 创建元任务列表
            }

            // 初始化其他任务列表
            for (int i = 0; i < jsTaskLists.length(); i++) { // 遍历 JSON 任务列表
                JSONObject object = jsTaskLists.getJSONObject(i); // 获取每个任务的 JSON 对象
                String gid = object.getString(GTaskStringUtils.GTASK_JSON_ID); // 获取 GID
                String name = object.getString(GTaskStringUtils.GTASK_JSON_NAME); // 获取名称

                // 判断任务名称是否以特定前缀开头
                if (name.startsWith(GTaskStringUtils.MIUI_FOLDER_PREFFIX) && !name.equals(GTaskStringUtils.MIUI_FOLDER_PREFFIX + GTaskStringUtils.FOLDER_META)) {
                    TaskList tasklist = new TaskList(); // 创建新的任务列表
                    tasklist.setContentByRemoteJSON(object); // 设置任务列表内容
                    mGTaskListHashMap.put(gid, tasklist); // 将任务列表存入哈希表
                    mGTaskHashMap.put(gid, tasklist); // 将任务列表存入任务哈希表

                    // 加载任务
                    JSONArray jsTasks = client.getTaskList(gid); // 获取任务列表的任务
                    for (int j = 0; j < jsTasks.length(); j++) { // 遍历任务
                        object = (JSONObject) jsTasks.getJSONObject(j); // 获取每个任务的 JSON 对象
                        gid = object.getString(GTaskStringUtils.GTASK_JSON_ID); // 获取 GID
                        Task task = new Task(); // 创建新的任务对象
                        task.setContentByRemoteJSON(object); // 设置任务内容
                        if (task.isWorthSaving()) { // 如果任务值得保存
                            task.setMetaInfo(mMetaHashMap.get(gid)); // 设置元信息
                            tasklist.addChildTask(task); // 将任务添加到任务列表中
                            mGTaskHashMap.put(gid, task); // 将任务存入任务哈希表
                        }
                    }
                }
            }
        } catch (JSONException e) { // 捕获 JSON 相关异常
            Log.e(TAG, e.toString()); // 记录错误信息
            e.printStackTrace(); // 打印堆栈跟踪
            throw new ActionFailureException("initGTaskList: handing JSONObject failed"); // 抛出操作失败异常
        }
    }

    // 同步内容方法
    private void syncContent() throws NetworkFailureException {
        int syncType; // 同步类型
        Cursor c = null; // 声明游标
        String gid; // 存储 GID
        Node node; // 声明节点

        mLocalDeleteIdMap.clear(); // 清空本地删除 ID 集合

        if (mCancelled) { // 如果已取消则返回
            return;
        }

        // 处理本地已删除的笔记
        try {
            c = mContentResolver.query(Notes.CONTENT_NOTE_URI, SqlNote.PROJECTION_NOTE, "(type<>? AND parent_id=?)", new String[] {
                    String.valueOf(Notes.TYPE_SYSTEM), String.valueOf(Notes.ID_TRASH_FOLER) // 查询非系统类型且不在回收站的笔记
            }, null);
            if (c != null) { // 如果查询成功
                while (c.moveToNext()) { // 遍历查询结果
                    gid = c.getString(SqlNote.GTASK_ID_COLUMN); // 获取 GID
                    node = mGTaskHashMap.get(gid); // 从哈希表中获取节点
                    if (node != null) { // 如果节点存在
                        mGTaskHashMap.remove(gid); // 从哈希表中移除节点
                        doContentSync(Node.SYNC_ACTION_DEL_REMOTE, node, c); // 执行删除同步
                    }

                    mLocalDeleteIdMap.add(c.getLong(SqlNote.ID_COLUMN)); // 添加到本地删除 ID 集合
                }
            } else {
                Log.w(TAG, "failed to query trash folder"); // 查询回收站失败
            }
        } finally { // 确保游标关闭
            if (c != null) {
                c.close(); // 关闭游标
                c = null;
            }
        }

        // 首先同步文件夹
        syncFolder();

        // 处理数据库中存在的笔记
        try {
            c = mContentResolver.query(Notes.CONTENT_NOTE_URI, SqlNote.PROJECTION_NOTE, "(type=? AND parent_id<>?)", new String[] {
                    String.valueOf(Notes.TYPE_NOTE), String.valueOf(Notes.ID_TRASH_FOLER) // 查询类型为笔记且不在回收站的记录
            }, NoteColumns.TYPE + " DESC"); // 根据类型降序排列
            if (c != null) { // 如果查询成功
                while (c.moveToNext()) { // 遍历查询结果
                    gid = c.getString(SqlNote.GTASK_ID_COLUMN); // 获取 GID
                    node = mGTaskHashMap.get(gid); // 从哈希表中获取节点
                    if (node != null) { // 如果节点存在
                        mGTaskHashMap.remove(gid); // 从哈希表中移除节点
                        mGidToNid.put(gid, c.getLong(SqlNote.ID_COLUMN)); // GID 到 NID 映射
                        mNidToGid.put(c.getLong(SqlNote.ID_COLUMN), gid); // NID 到 GID 映射
                        syncType = node.getSyncAction(c); // 获取同步类型
                    } else { // 如果节点不存在
                        if (c.getString(SqlNote.GTASK_ID_COLUMN).trim().length() == 0) {
                            // 本地添加
                            syncType = Node.SYNC_ACTION_ADD_REMOTE;
                        } else {
                            // 远程删除
                            syncType = Node.SYNC_ACTION_DEL_LOCAL;
                        }
                    }
                    doContentSync(syncType, node, c); // 执行内容同步
                }
            } else {
                Log.w(TAG, "failed to query existing note in database"); // 查询数据库中的笔记失败
            }

        } finally { // 确保游标关闭
            if (c != null) {
                c.close(); // 关闭游标
                c = null;
            }
        }

        // 遍历剩余项
        Iterator<Map.Entry<String, Node>> iter = mGTaskHashMap.entrySet().iterator();
        while (iter.hasNext()) { // 遍历哈希表
            Map.Entry<String, Node> entry = iter.next(); // 获取每个条目
            node = entry.getValue(); // 获取节点
            doContentSync(Node.SYNC_ACTION_ADD_LOCAL, node, null); // 执行本地添加同步
        }

        // mCancelled 可能由另一个线程设置，因此需要逐个检查
        // 清除本地删除表
        if (!mCancelled) {
            if (!DataUtils.batchDeleteNotes(mContentResolver, mLocalDeleteIdMap)) { // 执行批量删除
                throw new ActionFailureException("failed to batch-delete local deleted notes"); // 抛出操作失败异常
            }
        }

        // 刷新本地同步 ID
        if (!mCancelled) {
            GTaskClient.getInstance().commitUpdate(); // 提交更新
            refreshLocalSyncId(); // 刷新本地同步 ID
        }
    }

    // 同步文件夹的方法
    private void syncFolder() throws NetworkFailureException {
        Cursor c = null; // 声明游标
        String gid; // 存储 GID
        Node node; // 声明节点
        int syncType; // 同步类型

        if (mCancelled) { // 如果已取消则返回
            return;
        }

        // 处理根文件夹
        try {
            c = mContentResolver.query(ContentUris.withAppendedId(Notes.CONTENT_NOTE_URI, Notes.ID_ROOT_FOLDER), SqlNote.PROJECTION_NOTE, null, null, null); // 查询根文件夹
            if (c != null) { // 如果查询成功
                c.moveToNext(); // 移动到第一条记录
                gid = c.getString(SqlNote.GTASK_ID_COLUMN); // 获取 GID
                node = mGTaskHashMap.get(gid); // 从哈希表中获取节点
                if (node != null) { // 如果节点存在
                    mGTaskHashMap.remove(gid); // 从哈希表中移除节点
                    mGidToNid.put(gid, (long) Notes.ID_ROOT_FOLDER); // GID 到 NID 映射
                    mNidToGid.put((long) Notes.ID_ROOT_FOLDER, gid); // NID 到 GID 映射
                    // 对于系统文件夹，仅在必要时更新远程名称
                    if (!node.getName().equals(GTaskStringUtils.MIUI_FOLDER_PREFFIX + GTaskStringUtils.FOLDER_DEFAULT))
                        doContentSync(Node.SYNC_ACTION_UPDATE_REMOTE, node, c); // 执行更新同步
                } else {
                    doContentSync(Node.SYNC_ACTION_ADD_REMOTE, node, c); // 执行添加同步
                }
            } else {
                Log.w(TAG, "failed to query root folder"); // 查询根文件夹失败
            }
        } finally { // 确保游标关闭
            if (c != null) {
                c.close(); // 关闭游标
                c = null;
            }
        }

        // 处理通话记录文件夹
        try {
            c = mContentResolver.query(Notes.CONTENT_NOTE_URI, SqlNote.PROJECTION_NOTE, "(_id=?)", new String[] {
                    String.valueOf(Notes.ID_CALL_RECORD_FOLDER) // 根据 ID 查询通话记录文件夹
            }, null);
            if (c != null) { // 如果查询成功
                if (c.moveToNext()) { // 移动到第一条记录
                    gid = c.getString(SqlNote.GTASK_ID_COLUMN); // 获取 GID
                    node = mGTaskHashMap.get(gid); // 从哈希表中获取节点
                    if (node != null) { // 如果节点存在
                        mGTaskHashMap.remove(gid); // 从哈希表中移除节点
                        mGidToNid.put(gid, (long) Notes.ID_CALL_RECORD_FOLDER); // GID 到 NID 映射
                        mNidToGid.put((long) Notes.ID_CALL_RECORD_FOLDER, gid); // NID 到 GID 映射
                        // 对于系统文件夹，仅在必要时更新远程名称
                        if (!node.getName().equals(GTaskStringUtils.MIUI_FOLDER_PREFFIX + GTaskStringUtils.FOLDER_CALL_NOTE))
                            doContentSync(Node.SYNC_ACTION_UPDATE_REMOTE, node, c); // 执行更新同步
                    } else {
                        doContentSync(Node.SYNC_ACTION_ADD_REMOTE, node, c); // 执行添加同步
                    }
                }
            } else {
                Log.w(TAG, "failed to query call note folder"); // 查询通话记录文件夹失败
            }
        } finally { // 确保游标关闭
            if (c != null) {
                c.close(); // 关闭游标
                c = null;
            }
        }

        // 处理本地现有的文件夹
        try {
            c = mContentResolver.query(Notes.CONTENT_NOTE_URI, SqlNote.PROJECTION_NOTE, "(type=? AND parent_id<>?)", new String[] {
                    String.valueOf(Notes.TYPE_FOLDER), String.valueOf(Notes.ID_TRASH_FOLER) // 查询类型为文件夹且不在回收站的记录
            }, NoteColumns.TYPE + " DESC"); // 根据类型降序排列
            if (c != null) { // 如果查询成功
                while (c.moveToNext()) { // 遍历查询结果
                    gid = c.getString(SqlNote.GTASK_ID_COLUMN); // 获取 GID
                    node = mGTaskHashMap.get(gid); // 从哈希表中获取节点
                    if (node != null) { // 如果节点存在
                        mGTaskHashMap.remove(gid); // 从哈希表中移除节点
                        mGidToNid.put(gid, c.getLong(SqlNote.ID_COLUMN)); // GID 到 NID 映射
                        mNidToGid.put(c.getLong(SqlNote.ID_COLUMN), gid); // NID 到 GID 映射
                        syncType = node.getSyncAction(c); // 获取同步类型
                    } else {
                        if (c.getString(SqlNote.GTASK_ID_COLUMN).trim().length() == 0) {
                            // 本地添加
                            syncType = Node.SYNC_ACTION_ADD_REMOTE;
                        } else {
                            // 远程删除
                            syncType = Node.SYNC_ACTION_DEL_LOCAL;
                        }
                    }
                    doContentSync(syncType, node, c); // 执行内容同步
                }
            } else {
                Log.w(TAG, "failed to query existing folder"); // 查询已有文件夹失败
            }
        } finally { // 确保游标关闭
            if (c != null) {
                c.close(); // 关闭游标
                c = null;
            }
        }

        // 处理远程添加的文件夹
        Iterator<Map.Entry<String, TaskList>> iter = mGTaskListHashMap.entrySet().iterator();
        while (iter.hasNext()) { // 遍历哈希表
            Map.Entry<String, TaskList> entry = iter.next(); // 获取每个条目
            gid = entry.getKey(); // 获取 GID
            node = entry.getValue(); // 获取节点
            if (mGTaskHashMap.containsKey(gid)) { // 如果哈希表中包含该 GID
                mGTaskHashMap.remove(gid); // 从哈希表中移除该节点
                doContentSync(Node.SYNC_ACTION_ADD_LOCAL, node, null); // 执行本地添加同步
            }
        }

        if (!mCancelled) // 如果未取消
            GTaskClient.getInstance().commitUpdate(); // 提交更新
    }

    // 执行内容同步的具体操作
    private void doContentSync(int syncType, Node node, Cursor c) throws NetworkFailureException {
        if (mCancelled) { // 如果已取消则返回
            return;
        }

        MetaData meta; // 声明元数据
        switch (syncType) { // 根据同步类型执行不同操作
            case Node.SYNC_ACTION_ADD_LOCAL: // 本地添加
                addLocalNode(node); // 调用添加本地节点的方法
                break;
            case Node.SYNC_ACTION_ADD_REMOTE: // 远程添加
                addRemoteNode(node, c); // 调用添加远程节点的方法
                break;
            case Node.SYNC_ACTION_DEL_LOCAL: // 本地删除
                meta = mMetaHashMap.get(c.getString(SqlNote.GTASK_ID_COLUMN)); // 获取元数据
                if (meta != null) {
                    GTaskClient.getInstance().deleteNode(meta); // 删除远程节点
                }
                mLocalDeleteIdMap.add(c.getLong(SqlNote.ID_COLUMN)); // 添加到本地删除 ID 集合
                break;
            case Node.SYNC_ACTION_DEL_REMOTE: // 远程删除
                meta = mMetaHashMap.get(node.getGid()); // 获取元数据
                if (meta != null) {
                    GTaskClient.getInstance().deleteNode(meta); // 删除远程元节点
                }
                GTaskClient.getInstance().deleteNode(node); // 删除远程节点
                break;
            case Node.SYNC_ACTION_UPDATE_LOCAL: // 本地更新
                updateLocalNode(node, c); // 调用更新本地节点的方法
                break;
            case Node.SYNC_ACTION_UPDATE_REMOTE: // 远程更新
                updateRemoteNode(node, c); // 调用更新远程节点的方法
                break;
            case Node.SYNC_ACTION_UPDATE_CONFLICT: // 更新冲突
                // 将两种修改合并可能是个好主意
                // 目前简单地使用本地更新
                updateRemoteNode(node, c); // 调用更新远程节点的方法
                break;
            case Node.SYNC_ACTION_NONE: // 无需操作
                break;
            case Node.SYNC_ACTION_ERROR: // 错误操作类型
            default:
                throw new ActionFailureException("unknown sync action type"); // 抛出操作失败异常
        }
    }

    // 添加本地节点的方法
    private void addLocalNode(Node node) throws NetworkFailureException {
        if (mCancelled) { // 如果已取消则返回
            return;
        }

        SqlNote sqlNote; // 声明 SQL 笔记对象
        if (node instanceof TaskList) { // 如果节点是任务列表
            if (node.getName().equals(GTaskStringUtils.MIUI_FOLDER_PREFFIX + GTaskStringUtils.FOLDER_DEFAULT)) { // 如果是默认文件夹
                sqlNote = new SqlNote(mContext, Notes.ID_ROOT_FOLDER); // 创建 SQL 笔记对象
            } else if (node.getName().equals(GTaskStringUtils.MIUI_FOLDER_PREFFIX + GTaskStringUtils.FOLDER_CALL_NOTE)) { // 如果是通话记录文件夹
                sqlNote = new SqlNote(mContext, Notes.ID_CALL_RECORD_FOLDER); // 创建 SQL 笔记对象
            } else {
                sqlNote = new SqlNote(mContext); // 创建新的 SQL 笔记对象
                sqlNote.setContent(node.getLocalJSONFromContent()); // 设置内容
                sqlNote.setParentId(Notes.ID_ROOT_FOLDER); // 设置父 ID 为根文件夹
            }
        } else { // 如果节点是普通任务
            sqlNote = new SqlNote(mContext); // 创建新的 SQL 笔记对象
            JSONObject js = node.getLocalJSONFromContent(); // 获取任务的本地 JSON 内容
            try {
                if (js.has(GTaskStringUtils.META_HEAD_NOTE)) { // 如果 JSON 内容中有 HEAD_NOTE
                    JSONObject note = js.getJSONObject(GTaskStringUtils.META_HEAD_NOTE); // 获取 NOTE 对象
                    if (note.has(NoteColumns.ID)) { // 如果 NOTE 对象中有 ID
                        long id = note.getLong(NoteColumns.ID); // 获取 ID
                        if (DataUtils.existInNoteDatabase(mContentResolver, id)) { // 检查 ID 是否存在于数据库中
                            // ID 不可用，必须创建一个新 ID
                            note.remove(NoteColumns.ID); // 移除 ID
                        }
                    }
                }

                // 检查 JSON 对象是否包含 META_HEAD_DATA 字段
if (js.has(GTaskStringUtils.META_HEAD_DATA)) {
    JSONArray dataArray = js.getJSONArray(GTaskStringUtils.META_HEAD_DATA); // 获取数据数组
    for (int i = 0; i < dataArray.length(); i++) { // 遍历数据数组
        JSONObject data = dataArray.getJSONObject(i); // 获取每个数据的 JSON 对象
        if (data.has(DataColumns.ID)) { // 如果数据对象中包含 ID 字段
            long dataId = data.getLong(DataColumns.ID); // 获取数据的 ID
            if (DataUtils.existInDataDatabase(mContentResolver, dataId)) { // 检查数据 ID 是否在数据库中存在
                // 如果数据 ID 不可用，则移除 ID 字段
                data.remove(DataColumns.ID);
            }
        }
    }
}

// 捕获 JSON 异常
} catch (JSONException e) {
    Log.w(TAG, e.toString()); // 记录警告日志
    e.printStackTrace(); // 打印堆栈跟踪
}

// 设置 SQL 笔记对象的内容为 JSON
sqlNote.setContent(js);

// 获取父任务的 ID
Long parentId = mGidToNid.get(((Task) node).getParent().getGid());
if (parentId == null) { // 如果找不到父任务 ID
    Log.e(TAG, "cannot find task's parent id locally"); // 记录错误日志
    throw new ActionFailureException("cannot add local node"); // 抛出操作失败异常
}
sqlNote.setParentId(parentId.longValue()); // 设置 SQL 笔记的父 ID

// 创建本地节点
sqlNote.setGtaskId(node.getGid()); // 设置 GTask ID
sqlNote.commit(false); // 提交笔记内容，不同步到远程

// 更新 GID-NID 映射
mGidToNid.put(node.getGid(), sqlNote.getId()); // GID 到 NID 的映射
mNidToGid.put(sqlNote.getId(), node.getGid()); // NID 到 GID 的映射

// 更新元数据
updateRemoteMeta(node.getGid(), sqlNote); // 更新远程元数据

// 更新本地节点的方法
private void updateLocalNode(Node node, Cursor c) throws NetworkFailureException {
    if (mCancelled) { // 如果已取消则返回
        return;
    }

    SqlNote sqlNote; // 声明 SQL 笔记对象
    // 在本地更新笔记
    sqlNote = new SqlNote(mContext, c); // 根据游标初始化 SQL 笔记
    sqlNote.setContent(node.getLocalJSONFromContent()); // 设置内容为节点的本地 JSON

    // 获取父 ID
    Long parentId = (node instanceof Task) ? mGidToNid.get(((Task) node).getParent().getGid())
            : new Long(Notes.ID_ROOT_FOLDER); // 如果节点是任务，则获取任务的父 ID；否则使用根文件夹 ID
    if (parentId == null) { // 如果找不到父任务 ID
        Log.e(TAG, "cannot find task's parent id locally"); // 记录错误日志
        throw new ActionFailureException("cannot update local node"); // 抛出操作失败异常
    }
    sqlNote.setParentId(parentId.longValue()); // 设置 SQL 笔记的父 ID
    sqlNote.commit(true); // 提交笔记内容并同步到远程

    // 更新元信息
    updateRemoteMeta(node.getGid(), sqlNote); // 更新远程元数据
}

// 添加远程节点的方法
private void addRemoteNode(Node node, Cursor c) throws NetworkFailureException {
    if (mCancelled) { // 如果已取消则返回
        return;
    }

    SqlNote sqlNote = new SqlNote(mContext, c); // 根据游标初始化 SQL 笔记
    Node n; // 声明节点

    // 更新远程内容
    if (sqlNote.isNoteType()) { // 如果是笔记类型
        Task task = new Task(); // 创建任务对象
        task.setContentByLocalJSON(sqlNote.getContent()); // 从 SQL 笔记获取本地内容

        String parentGid = mNidToGid.get(sqlNote.getParentId()); // 获取父任务的 GID
        if (parentGid == null) { // 如果找不到父任务 GID
            Log.e(TAG, "cannot find task's parent tasklist"); // 记录错误日志
            throw new ActionFailureException("cannot add remote task"); // 抛出操作失败异常
        }
        mGTaskListHashMap.get(parentGid).addChildTask(task); // 将任务添加到父任务列表

        GTaskClient.getInstance().createTask(task); // 在远程创建任务
        n = (Node) task; // 将任务赋值给节点 n

        // 添加元数据
        updateRemoteMeta(task.getGid(), sqlNote); // 更新远程元数据
    } else { // 如果是文件夹类型
        TaskList tasklist = null;

        // 如果文件夹已经存在，则跳过
        String folderName = GTaskStringUtils.MIUI_FOLDER_PREFFIX; // 设置文件夹名称前缀
        if (sqlNote.getId() == Notes.ID_ROOT_FOLDER) // 根文件夹
            folderName += GTaskStringUtils.FOLDER_DEFAULT;
        else if (sqlNote.getId() == Notes.ID_CALL_RECORD_FOLDER) // 通话记录文件夹
            folderName += GTaskStringUtils.FOLDER_CALL_NOTE;
        else // 其他文件夹
            folderName += sqlNote.getSnippet();

        // 检查现有任务列表中是否已存在该文件夹
        Iterator<Map.Entry<String, TaskList>> iter = mGTaskListHashMap.entrySet().iterator();
        while (iter.hasNext()) { // 遍历任务列表
            Map.Entry<String, TaskList> entry = iter.next(); // 获取每个条目
            String gid = entry.getKey(); // 获取 GID
            TaskList list = entry.getValue(); // 获取任务列表

            if (list.getName().equals(folderName)) { // 如果找到匹配的文件夹名称
                tasklist = list; // 获取文件夹
                if (mGTaskHashMap.containsKey(gid)) { // 如果任务哈希表中包含该 GID
                    mGTaskHashMap.remove(gid); // 移除该 GID
                }
                break; // 退出循环
            }
        }

        // 如果没有找到匹配的文件夹，则创建新的文件夹
        if (tasklist == null) {
            tasklist = new TaskList(); // 创建新的任务列表
            tasklist.setContentByLocalJSON(sqlNote.getContent()); // 设置任务列表内容
            GTaskClient.getInstance().createTaskList(tasklist); // 在远程创建任务列表
            mGTaskListHashMap.put(tasklist.getGid(), tasklist); // 将任务列表添加到哈希表
        }
        n = (Node) tasklist; // 将任务列表赋值给节点 n
    }

    // 更新本地笔记
    sqlNote.setGtaskId(n.getGid()); // 设置 GTask ID
    sqlNote.commit(false); // 提交笔记内容，不同步到远程
    sqlNote.resetLocalModified(); // 重置本地修改标记
    sqlNote.commit(true); // 提交并同步到远程

    // 更新 GID-ID 映射
    mGidToNid.put(n.getGid(), sqlNote.getId()); // GID 到 NID 的映射
    mNidToGid.put(sqlNote.getId(), n.getGid()); // NID 到 GID 的映射
}

// 更新远程节点的方法
private void updateRemoteNode(Node node, Cursor c) throws NetworkFailureException {
    if (mCancelled) { // 如果已取消则返回
        return;
    }

    SqlNote sqlNote = new SqlNote(mContext, c); // 根据游标初始化 SQL 笔记

    // 更新远程内容
    node.setContentByLocalJSON(sqlNote.getContent()); // 设置节点内容
    GTaskClient.getInstance().addUpdateNode(node); // 添加更新节点

    // 更新元数据
    updateRemoteMeta(node.getGid(), sqlNote); // 更新远程元数据

    // 如果需要，移动任务
    if (sqlNote.isNoteType()) { // 如果节点是笔记类型
        Task task = (Task) node; // 将节点转换为任务
        TaskList preParentList = task.getParent(); // 获取原父任务列表

        String curParentGid = mNidToGid.get(sqlNote.getParentId()); // 获取当前父任务 GID
        if (curParentGid == null) { // 如果找不到父任务 GID
            Log.e(TAG, "cannot find task's parent tasklist"); // 记录错误日志
            throw new ActionFailureException("cannot update remote task"); // 抛出操作失败异常
        }
        TaskList curParentList = mGTaskListHashMap.get(curParentGid); // 获取当前父任务列表

        // 如果前一个父列表与当前父列表不同，则移动任务
        if (preParentList != curParentList) {
            preParentList.removeChildTask(task); // 从原父列表中移除任务
            curParentList.addChildTask(task); // 将任务添加到当前父列表
            GTaskClient.getInstance().moveTask(task, preParentList, curParentList); // 移动任务至远程
        }
    }

    // 清除本地修改标记
    sqlNote.resetLocalModified(); // 重置本地修改标记
    sqlNote.commit(true); // 提交并同步到远程
}

// 更新远程元数据的方法
private void updateRemoteMeta(String gid, SqlNote sqlNote) throws NetworkFailureException {
    if (sqlNote != null && sqlNote.isNoteType()) { // 如果 SQL 笔记不为 null 且是笔记类型
        MetaData metaData = mMetaHashMap.get(gid); // 获取元数据对象
        if (metaData != null) { // 如果元数据存在
            metaData.setMeta(gid, sqlNote.getContent()); // 更新元数据内容
            GTaskClient.getInstance().addUpdateNode(metaData); // 在远程更新元数据
        } else { // 如果元数据不存在
            metaData = new MetaData(); // 创建新的元数据对象
            metaData.setMeta(gid, sqlNote.getContent()); // 设置元数据内容
            mMetaList.addChildTask(metaData); // 将元数据添加到元数据列表
            mMetaHashMap.put(gid, metaData); // 更新元数据哈希表
            GTaskClient.getInstance().createTask(metaData); // 在远程创建元数据
        }
    }
}

// 刷新本地同步 ID 的方法
private void refreshLocalSyncId() throws NetworkFailureException {
    if (mCancelled) { // 如果已取消则返回
        return;
    }

    // 获取最新的 GTask 列表
    mGTaskHashMap.clear(); // 清空任务哈希表
    mGTaskListHashMap.clear(); // 清空任务列表哈希表
    mMetaHashMap.clear(); // 清空元数据哈希表
    initGTaskList(); // 初始化 GTask 列表

    Cursor c = null; // 声明游标
    try {
        c = mContentResolver.query(Notes.CONTENT_NOTE_URI, SqlNote.PROJECTION_NOTE,
                "(type<>? AND parent_id<>?)", new String[] {
                        String.valueOf(Notes.TYPE_SYSTEM), String.valueOf(Notes.ID_TRASH_FOLER) // 查询非系统类型且不在回收站的笔记
                }, NoteColumns.TYPE + " DESC"); // 根据类型降序排列
        if (c != null) { // 如果查询成功
            while (c.moveToNext()) { // 遍历查询结果
                String gid = c.getString(SqlNote.GTASK_ID_COLUMN); // 获取 GID
                Node node = mGTaskHashMap.get(gid); // 从哈希表中获取节点
                if (node != null) { // 如果节点存在
                    mGTaskHashMap.remove(gid); // 从哈希表中移除节点
                    ContentValues values = new ContentValues(); // 创建内容值对象
                    values.put(NoteColumns.SYNC_ID, node.getLastModified()); // 更新同步 ID
                    mContentResolver.update(ContentUris.withAppendedId(Notes.CONTENT_NOTE_URI,
                            c.getLong(SqlNote.ID_COLUMN)), values, null, null); // 更新内容
                } else {
                    Log.e(TAG, "something is missed"); // 记录错误日志
                    throw new ActionFailureException("some local items don't have gid after sync"); // 抛出操作失败异常
                }
            }
        } else {
            Log.w(TAG, "failed to query local note to refresh sync id"); // 查询本地笔记以刷新同步 ID 失败
        }
    } finally { // 确保游标关闭
        if (c != null) {
            c.close(); // 关闭游标
            c = null;
        }
    }
}

// 获取同步帐户信息的方法
public String getSyncAccount() {
    return GTaskClient.getInstance().getSyncAccount().name; // 返回同步帐户名
}

// 取消同步操作的方法
public void cancelSync() {
    mCancelled = true; // 设置取消标志
}
