/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.gtask.remote;

// 导入必要的Android类
import android.app.Activity;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.os.IBinder;

// 定义GTaskSyncService类，继承自Service
public class GTaskSyncService extends Service {
    // 定义同步动作的字符串名称
    public final static String ACTION_STRING_NAME = "sync_action_type";

    // 定义开始同步的动作标识
    public final static int ACTION_START_SYNC = 0;

    // 定义取消同步的动作标识
    public final static int ACTION_CANCEL_SYNC = 1;

    // 定义无效动作的标识
    public final static int ACTION_INVALID = 2;

    // 定义广播名称，用于发送同步服务的广播
    public final static String GTASK_SERVICE_BROADCAST_NAME = "net.micode.notes.gtask.remote.gtask_sync_service";

    // 定义广播中表示是否正在同步的键
    public final static String GTASK_SERVICE_BROADCAST_IS_SYNCING = "isSyncing";

    // 定义广播中表示进度消息的键
    public final static String GTASK_SERVICE_BROADCAST_PROGRESS_MSG = "progressMsg";

    // 静态变量，表示当前的异步同步任务
    private static GTaskASyncTask mSyncTask = null;

    // 静态变量，保存同步进度消息
    private static String mSyncProgress = "";

    // 启动同步的方法
    private void startSync() {
        // 如果当前没有正在进行的同步任务
        if (mSyncTask == null) {
            // 创建新的异步同步任务
            mSyncTask = new GTaskASyncTask(this, new GTaskASyncTask.OnCompleteListener() {
                // 同步完成时的回调
                public void onComplete() {
                    mSyncTask = null; // 清空任务
                    sendBroadcast(""); // 发送广播
                    stopSelf(); // 停止服务
                }
            });
            sendBroadcast(""); // 发送广播，通知开始同步
            mSyncTask.execute(); // 执行同步任务
        }
    }

    // 取消同步的方法
    private void cancelSync() {
        // 如果有同步任务在运行
        if (mSyncTask != null) {
            mSyncTask.cancelSync(); // 取消同步
        }
    }

    // 服务创建时调用的方法
    @Override
    public void onCreate() {
        mSyncTask = null; // 初始化同步任务为null
    }

    // 当服务被启动时调用的方法
    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        Bundle bundle = intent.getExtras(); // 获取传入的意图数据
        // 如果意图中包含动作标识
        if (bundle != null && bundle.containsKey(ACTION_STRING_NAME)) {
            // 根据动作标识进行处理
            switch (bundle.getInt(ACTION_STRING_NAME, ACTION_INVALID)) {
                case ACTION_START_SYNC: // 开始同步
                    startSync();
                    break;
                case ACTION_CANCEL_SYNC: // 取消同步
                    cancelSync();
                    break;
                default:
                    break;
            }
            return START_STICKY; // 表示服务在被系统杀死后应再次启动
        }
        return super.onStartCommand(intent, flags, startId); // 调用父类的方法
    }

    // 系统内存不足时调用的方法
    @Override
    public void onLowMemory() {
        // 如果有正在运行的同步任务，取消它
        if (mSyncTask != null) {
            mSyncTask.cancelSync();
        }
    }

    // 绑定服务的方法，返回null表示不支持绑定
    public IBinder onBind(Intent intent) {
        return null;
    }

    // 发送广播的方法，携带进度消息
    public void sendBroadcast(String msg) {
        mSyncProgress = msg; // 更新进度消息
        Intent intent = new Intent(GTASK_SERVICE_BROADCAST_NAME); // 创建广播意图
        intent.putExtra(GTASK_SERVICE_BROADCAST_IS_SYNCING, mSyncTask != null); // 添加正在同步的状态
        intent.putExtra(GTASK_SERVICE_BROADCAST_PROGRESS_MSG, msg); // 添加进度消息
        sendBroadcast(intent); // 发送广播
    }

    // 启动同步的静态方法
    public static void startSync(Activity activity) {
        GTaskManager.getInstance().setActivityContext(activity); // 设置上下文
        Intent intent = new Intent(activity, GTaskSyncService.class); // 创建启动服务的意图
        intent.putExtra(GTaskSyncService.ACTION_STRING_NAME, GTaskSyncService.ACTION_START_SYNC); // 添加动作标识
        activity.startService(intent); // 启动服务
    }

    // 取消同步的静态方法
    public static void cancelSync(Context context) {
        Intent intent = new Intent(context, GTaskSyncService.class); // 创建取消服务的意图
        intent.putExtra(GTaskSyncService.ACTION_STRING_NAME, GTaskSyncService.ACTION_CANCEL_SYNC); // 添加动作标识
        context.startService(intent); // 启动服务
    }

    // 静态方法，用于判断当前是否正在同步
    public static boolean isSyncing() {
        return mSyncTask != null; // 如果任务不为null，则表示正在同步
    }

    // 静态方法，用于获取当前的进度字符串
    public static String getProgressString() {
        return mSyncProgress; // 返回同步进度消息
    }
}

