/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.tool;

import android.content.ContentProviderOperation;
import android.content.ContentProviderResult;
import android.content.ContentResolver;
import android.content.ContentUris;
import android.content.ContentValues;
import android.content.OperationApplicationException;
import android.database.Cursor;
import android.os.RemoteException;
import android.util.Log;

import net.micode.notes.data.Notes;
import net.micode.notes.data.Notes.CallNote;
import net.micode.notes.data.Notes.NoteColumns;
import net.micode.notes.ui.NotesListAdapter.AppWidgetAttribute;

import java.util.ArrayList;
import java.util.HashSet;

/**
 * 工具类，提供对笔记数据的操作方法。
 */
public class DataUtils {
    public static final String TAG = "DataUtils"; // 定义日志标签

    /**
     * 批量删除笔记。
     * @param resolver ContentResolver实例，用于访问数据。
     * @param ids 要删除的笔记ID集合。
     * @return 如果删除成功返回true，否则返回false。
     */
    public static boolean batchDeleteNotes(ContentResolver resolver, HashSet<Long> ids) {
        if (ids == null) { // 检查传入的ID集合是否为空
            Log.d(TAG, "the ids is null");
            return true;
        }
        if (ids.size() == 0) { // 检查ID集合是否为空
            Log.d(TAG, "no id is in the hashset");
            return true;
        }

        ArrayList<ContentProviderOperation> operationList = new ArrayList<ContentProviderOperation>(); // 操作列表，用于批量操作
        for (long id : ids) {
            if(id == Notes.ID_ROOT_FOLDER) { // 检查是否尝试删除根文件夹，这是不允许的
                Log.e(TAG, "Don't delete system folder root");
                continue;
            }
            ContentProviderOperation.Builder builder = ContentProviderOperation
                    .newDelete(ContentUris.withAppendedId(Notes.CONTENT_NOTE_URI, id)); // 构建删除操作
            operationList.add(builder.build()); // 将删除操作添加到操作列表
        }
        try {
            ContentProviderResult[] results = resolver.applyBatch(Notes.AUTHORITY, operationList); // 执行批量操作
            if (results == null || results.length == 0 || results[0] == null) { // 检查操作结果
                Log.d(TAG, "delete notes failed, ids:" + ids.toString());
                return false;
            }
            return true;
        } catch (RemoteException e) { // 捕获远程异常
            Log.e(TAG, String.format("%s: %s", e.toString(), e.getMessage()));
        } catch (OperationApplicationException e) { // 捕获操作应用异常
            Log.e(TAG, String.format("%s: %s", e.toString(), e.getMessage()));
        }
        return false; // 发生异常时返回false
    }

    /**
     * 将笔记移动到另一个文件夹。
     * @param resolver ContentResolver实例，用于访问数据。
     * @param id 要移动的笔记ID。
     * @param srcFolderId 当前文件夹ID。
     * @param desFolderId 目标文件夹ID。
     */
    public static void moveNoteToFoler(ContentResolver resolver, long id, long srcFolderId, long desFolderId) {
        ContentValues values = new ContentValues(); // 创建ContentValues对象，用于更新数据
        values.put(NoteColumns.PARENT_ID, desFolderId); // 设置新的父ID，即目标文件夹ID
        values.put(NoteColumns.ORIGIN_PARENT_ID, srcFolderId); // 设置原始的父ID，即当前文件夹ID
        values.put(NoteColumns.LOCAL_MODIFIED, 1); // 标记笔记为本地修改
        resolver.update(ContentUris.withAppendedId(Notes.CONTENT_NOTE_URI, id), values, null, null); // 更新笔记数据
    }

    /**
     * 批量将笔记移动到指定文件夹。
     * @param resolver ContentResolver实例，用于访问数据。
     * @param ids 要移动的笔记ID集合。
     * @param folderId 目标文件夹ID。
     * @return 如果移动成功返回true，否则返回false。
     */
    public static boolean batchMoveToFolder(ContentResolver resolver, HashSet<Long> ids,
            long folderId) {
        if (ids == null) { // 检查传入的ID集合是否为空
            Log.d(TAG, "the ids is null");
            return true;
        }

        ArrayList<ContentProviderOperation> operationList = new ArrayList<ContentProviderOperation>(); // 操作列表，用于批量操作
        for (long id : ids) {
            ContentProviderOperation.Builder builder = ContentProviderOperation
                    .newUpdate(ContentUris.withAppendedId(Notes.CONTENT_NOTE_URI, id)); // 构建更新操作
            builder.withValue(NoteColumns.PARENT_ID, folderId); // 设置新的父ID，即目标文件夹ID
            builder.withValue(NoteColumns.LOCAL_MODIFIED, 1); // 标记笔记为本地修改
            operationList.add(builder.build()); // 将更新操作添加到操作列表
        }

        try {
            ContentProviderResult[] results = resolver.applyBatch(Notes.AUTHORITY, operationList); // 执行批量操作
            if (results == null || results.length == 0 || results[0] == null) { // 检查操作结果
                Log.d(TAG, "delete notes failed, ids:" + ids.toString());
                return false;
            }
            return true;
        } catch (RemoteException e) { // 捕获远程异常
            Log.e(TAG, String.format("%s: %s", e.toString(), e.getMessage()));
        } catch (OperationApplicationException e) { // 捕获操作应用异常
            Log.e(TAG, String.format("%s: %s", e.toString(), e.getMessage()));
        }
        return false; // 发生异常时返回false
    }

    /**
     * 获取用户文件夹的数量（不包括系统文件夹）。
     * @param resolver ContentResolver实例，用于访问数据。
     * @return 用户文件夹的数量。
     */
    public static int getUserFolderCount(ContentResolver resolver) {
        Cursor cursor = resolver.query(Notes.CONTENT_NOTE_URI, // 查询笔记内容URI
                new String[] { "COUNT(*)" }, // 查询计数
                NoteColumns.TYPE + "=? AND " + NoteColumns.PARENT_ID + "<>?", // 查询条件，类型为文件夹且不是垃圾箱
                new String[] { String.valueOf(Notes.TYPE_FOLDER), String.valueOf(Notes.ID_TRASH_FOLER)}, // 查询参数
                null);

        int count = 0;
        if (cursor != null) { // 检查游标是否为空
            if (cursor.moveToFirst()) { // 移动到游标的第一行
                try {
                    count = cursor.getInt(0); // 获取文件夹数量
                } catch (IndexOutOfBoundsException e) { // 捕获索引越界异常
                    Log.e(TAG, "get folder count failed:" + e.toString());
                } finally {
                    cursor.close(); // 关闭游标
                }
            }
        }
        return count; // 返回文件夹数量
    }

    /**
     * 检查笔记在数据库中是否可见。
     * @param resolver ContentResolver实例，用于访问数据。
     * @param noteId 笔记ID。
     * @param type 笔记类型。
     * @return 如果笔记可见返回true，否则返回false。
     */
    public static boolean visibleInNoteDatabase(ContentResolver resolver, long noteId, int type) {
        Cursor cursor = resolver.query(ContentUris.withAppendedId(Notes.CONTENT_NOTE_URI, noteId), // 查询笔记URI
                null, // 无特定列
                NoteColumns.TYPE + "=? AND " + NoteColumns.PARENT_ID + "<>" + Notes.ID_TRASH_FOLER, // 查询条件，类型匹配且不在垃圾箱
                new String[] {String.valueOf(type)}, // 查询参数
                null);

        boolean exist = false;
        if (cursor != null) { // 检查游标是否为空
            if (cursor.getCount() > 0) { // 检查查询结果数量
                exist = true; // 设置存在标志为true
            }
            cursor.close(); // 关闭游标
        }
        return exist; // 返回笔记是否存在
    }

    /**
     * 检查笔记在数据库中是否存在。
     * @param resolver ContentResolver实例，用于访问数据。
     * @param noteId 笔记ID。
     * @return 如果笔记存在返回true，否则返回false。
     */
    public static boolean existInNoteDatabase(ContentResolver resolver, long noteId)