/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.tool;

import android.content.Context;
import android.preference.PreferenceManager;

import net.micode.notes.R;
import net.micode.notes.ui.NotesPreferenceActivity;

/**
 * 资源解析工具类，用于管理不同颜色和字体大小的资源ID。
 */
public class ResourceParser {

    // 定义颜色常量
    public static final int YELLOW           = 0;//黄色为0
    public static final int BLUE             = 1;//蓝色为1
    public static final int WHITE            = 2;//白色为2
    public static final int GREEN            = 3;//绿色为3
    public static final int RED              = 4;//红色为4

    // 定义默认背景颜色常量
    public static final int BG_DEFAULT_COLOR = YELLOW;

    // 定义字体大小常量
    public static final int TEXT_SMALL       = 0;
    public static final int TEXT_MEDIUM      = 1;
    public static final int TEXT_LARGE       = 2;
    public static final int TEXT_SUPER       = 3;

    // 定义默认字体大小常量
    public static final int BG_DEFAULT_FONT_SIZE = TEXT_MEDIUM;

    /**
     * 笔记背景资源类，用于获取不同颜色的笔记背景资源ID。
     */
    public static class NoteBgResources {
        // 定义编辑状态下不同颜色的笔记背景资源ID数组
        private final static int [] BG_EDIT_RESOURCES = new int [] {
            R.drawable.edit_yellow,
            R.drawable.edit_blue,
            R.drawable.edit_white,
            R.drawable.edit_green,
            R.drawable.edit_red
        };

        // 定义编辑状态下不同颜色的笔记标题背景资源ID数组
        private final static int [] BG_EDIT_TITLE_RESOURCES = new int [] {
            R.drawable.edit_title_yellow,
            R.drawable.edit_title_blue,
            R.drawable.edit_title_white,
            R.drawable.edit_title_green,
            R.drawable.edit_title_red
        };

        // 获取编辑状态下指定颜色的笔记背景资源ID
        public static int getNoteBgResource(int id) {
            return BG_EDIT_RESOURCES[id];
        }

        // 获取编辑状态下指定颜色的笔记标题背景资源ID
        public static int getNoteTitleBgResource(int id) {
            return BG_EDIT_TITLE_RESOURCES[id];
        }
    }

    /**
     * 获取默认的背景ID。
     * 如果设置了自定义背景颜色，则随机返回一个背景ID，否则返回默认背景ID。
     */
    public static int getDefaultBgId(Context context) {
        if (PreferenceManager.getDefaultSharedPreferences(context).getBoolean(
                NotesPreferenceActivity.PREFERENCE_SET_BG_COLOR_KEY, false)) {
            return (int) (Math.random() * NoteBgResources.BG_EDIT_RESOURCES.length);
        } else {
            return BG_DEFAULT_COLOR;
        }
    }

    /**
     * 笔记项背景资源类，用于获取不同位置（首项、末项、单项、普通项）的笔记背景资源ID。
     */
    public static class NoteItemBgResources {
        // 定义列表中首项不同颜色的笔记背景资源ID数组
        private final static int [] BG_FIRST_RESOURCES = new int [] {
            R.drawable.list_yellow_up,
            R.drawable.list_blue_up,
            R.drawable.list_white_up,
            R.drawable.list_green_up,
            R.drawable.list_red_up
        };

        // 定义列表中普通项不同颜色的笔记背景资源ID数组
        private final static int [] BG_NORMAL_RESOURCES = new int [] {
            R.drawable.list_yellow_middle,
            R.drawable.list_blue_middle,
            R.drawable.list_white_middle,
            R.drawable.list_green_middle,
            R.drawable.list_red_middle
        };

        // 定义列表中末项不同颜色的笔记背景资源ID数组
        private final static int [] BG_LAST_RESOURCES = new int [] {
            R.drawable.list_yellow_down,
            R.drawable.list_blue_down,
            R.drawable.list_white_down,
            R.drawable.list_green_down,
            R.drawable.list_red_down,
        };

        // 定义列表中单项不同颜色的笔记背景资源ID数组
        private final static int [] BG_SINGLE_RESOURCES = new int [] {
            R.drawable.list_yellow_single,
            R.drawable.list_blue_single,
            R.drawable.list_white_single,
            R.drawable.list_green_single,
            R.drawable.list_red_single
        };

        // 获取列表中首项指定颜色的笔记背景资源ID
        public static int getNoteBgFirstRes(int id) {
            return BG_FIRST_RESOURCES[id];
        }

        // 获取列表中末项指定颜色的笔记背景资源ID
        public static int getNoteBgLastRes(int id) {
            return BG_LAST_RESOURCES[id];
        }

        // 获取列表中单项指定颜色的笔记背景资源ID
        public static int getNoteBgSingleRes(int id) {
            return BG_SINGLE_RESOURCES[id];
        }

        // 获取列表中普通项指定颜色的笔记背景资源ID
        public static int getNoteBgNormalRes(int id) {
            return BG_NORMAL_RESOURCES[id];
        }

        // 获取文件夹背景资源ID
        public static int getFolderBgRes() {
            return R.drawable.list_folder;
        }
    }

    /**
     * 部件背景资源类，用于获取不同大小（2x2和4x4）的部件背景资源ID。
     */
    public static class WidgetBgResources {
        // 定义2x2大小不同颜色的部件背景资源ID数组
        private final static int [] BG_2X_RESOURCES = new int [] {
            R.drawable.widget_2x_yellow,
            R.drawable.widget_2x_blue,
            R.drawable.widget_2x_white,
            R.drawable.widget_2x_green,
            R.drawable.widget_2x_red,
        };

        // 获取2x2大小指定颜色的部件背景资源ID
        public static int getWidget2xBgResource(int id) {
            return BG_2X_RESOURCES[id];
        }

        // 定义4x4大小不同颜色的部件背景资源ID数组
        private final static int [] BG_4X_RESOURCES = new int [] {
            R.drawable.widget_4x_yellow,
            R.drawable.widget_4x_blue,
            R.drawable.widget_4x_white,
            R.drawable.widget_4x_green,
            R.drawable.widget_4x_red
        };

        // 获取4x4大小指定颜色的部件背景资源ID
        public static int getWidget4xBgResource(int id) {
            return BG_4X_RESOURCES[id];
        }
    }

    /**
     * 文本外观资源类，用于获取不同大小的文本外观资源ID。
     */
    public static class TextAppearanceResources {
        // 定义不同大小的文本外观资源ID数组
        private final static int [] TEXTAPPEARANCE_RESOURCES = new int [] {
            R.style.TextAppearanceNormal,
            R.style.TextAppearanceMedium,
            R.style.TextAppearanceLarge,
            R.style.TextAppearanceSuper
        };

        // 获取指定大小的文本外观资源ID
        public static int getTexAppearanceResource(int id) {
            /**
             * HACKME: Fix bug of store the resource id in shared preference.
             * The id may larger than the length of resources, in this case,
             * return the {@link ResourceParser#BG_DEFAULT_FONT_SIZE}
             */
            if (id >= TEXTAPPEARANCE_RESOURCES.length) {
                return BG_DEFAULT_FONT_SIZE;
            }
            return TEXTAPPEARANCE_RESOURCES[id];
        }

        // 获取文本外观资源数组的大小
        public static int getResourcesSize() {
            return TEXTAPPEARANCE_RESOURCES.length;
        }
    }
}
package net.micode.notes.ui; // 定义包名

import android.content.Context; // 导入上下文类
import android.database.Cursor; // 导入游标类
import android.view.View; // 导入视图类
import android.view.ViewGroup; // 导入视图组类
import android.widget.CursorAdapter; // 导入游标适配器类
import android.widget.LinearLayout; // 导入线性布局类
import android.widget.TextView; // 导入文本视图类

import net.micode.notes.R; // 导入资源类
import net.micode.notes.data.Notes; // 导入笔记数据类
import net.micode.notes.data.Notes.NoteColumns; // 导入笔记列类

public class FoldersListAdapter extends CursorAdapter { // 定义文件夹列表适配器类，继承自 CursorAdapter
    public static final String [] PROJECTION = { // 定义需要查询的字段数组
        NoteColumns.ID, // 笔记 ID 列
        NoteColumns.SNIPPET // 笔记摘要列
    };

    public static final int ID_COLUMN   = 0; // ID 列索引
    public static final int NAME_COLUMN = 1; // 名称列索引

    // 构造函数，接受上下文和游标作为参数
    public FoldersListAdapter(Context context, Cursor c) {
        super(context, c); // 调用父类构造函数
        // TODO Auto-generated constructor stub
    }

    // 创建新的视图
    @Override
    public View newView(Context context, Cursor cursor, ViewGroup parent) {
        return new FolderListItem(context); // 返回新的文件夹列表项视图
    }

    // 绑定视图与数据
    @Override
    public void bindView(View view, Context context, Cursor cursor) {
        if (view instanceof FolderListItem) { // 如果视图是 FolderListItem 的实例
            // 获取文件夹名称，判断是否为根文件夹
            String folderName = (cursor.getLong(ID_COLUMN) == Notes.ID_ROOT_FOLDER) ? context
                    .getString(R.string.menu_move_parent_folder) : cursor.getString(NAME_COLUMN);
            ((FolderListItem) view).bind(folderName); // 绑定文件夹名称到视图
        }
    }

    // 根据位置获取文件夹名称
    public String getFolderName(Context context, int position) {
        Cursor cursor = (Cursor) getItem(position); // 获取指定位置的游标
        // 判断是否为根文件夹，返回相应的名称
        return (cursor.getLong(ID_COLUMN) == Notes.ID_ROOT_FOLDER) ? context
                .getString(R.string.menu_move_parent_folder) : cursor.getString(NAME_COLUMN);
    }

    // 内部类，表示文件夹列表项
    private class FolderListItem extends LinearLayout { // 继承自 LinearLayout
        private TextView mName; // 存储文件夹名称的文本视图

        // 构造函数，接受上下文作为参数
        public FolderListItem(Context context) {
            super(context); // 调用父类构造函数
            inflate(context, R.layout.folder_list_item, this); // 加载布局文件
            mName = (TextView) findViewById(R.id.tv_folder_name); // 初始化文件夹名称文本视图
        }

        // 绑定文件夹名称到文本视图
        public void bind(String name) {
            mName.setText(name); // 设置文本视图的文本为文件夹名称
        }
    }
package net.micode.notes.ui; // 定义包名

import android.content.Context; // 导入上下文类
import android.database.Cursor; // 导入游标类
import android.view.View; // 导入视图类
import android.view.ViewGroup; // 导入视图组类
import android.widget.CursorAdapter; // 导入游标适配器类
import android.widget.LinearLayout; // 导入线性布局类
import android.widget.TextView; // 导入文本视图类

import net.micode.notes.R; // 导入资源类
import net.micode.notes.data.Notes; // 导入笔记数据类
import net.micode.notes.data.Notes.NoteColumns; // 导入笔记列类

public class FoldersListAdapter extends CursorAdapter { // 定义文件夹列表适配器类，继承自 CursorAdapter
    public static final String [] PROJECTION = { // 定义需要查询的字段数组
        NoteColumns.ID, // 笔记 ID 列
        NoteColumns.SNIPPET // 笔记摘要列
    };

    public static final int ID_COLUMN   = 0; // ID 列索引
    public static final int NAME_COLUMN = 1; // 名称列索引

    // 构造函数，接受上下文和游标作为参数
    public FoldersListAdapter(Context context, Cursor c) {
        super(context, c); // 调用父类构造函数
        // TODO Auto-generated constructor stub
    }

    // 创建新的视图
    @Override
    public View newView(Context context, Cursor cursor, ViewGroup parent) {
        return new FolderListItem(context); // 返回新的文件夹列表项视图
    }

    // 绑定视图与数据
    @Override
    public void bindView(View view, Context context, Cursor cursor) {
        if (view instanceof FolderListItem) { // 如果视图是 FolderListItem 的实例
            // 获取文件夹名称，判断是否为根文件夹
            String folderName = (cursor.getLong(ID_COLUMN) == Notes.ID_ROOT_FOLDER) ? context
                    .getString(R.string.menu_move_parent_folder) : cursor.getString(NAME_COLUMN);
            ((FolderListItem) view).bind(folderName); // 绑定文件夹名称到视图
        }
    }

    // 根据位置获取文件夹名称
    public String getFolderName(Context context, int position) {
        Cursor cursor = (Cursor) getItem(position); // 获取指定位置的游标
        // 判断是否为根文件夹，返回相应的名称
        return (cursor.getLong(ID_COLUMN) == Notes.ID_ROOT_FOLDER) ? context
                .getString(R.string.menu_move_parent_folder) : cursor.getString(NAME_COLUMN);
    }

    // 内部类，表示文件夹列表项
    private class FolderListItem extends LinearLayout { // 继承自 LinearLayout
        private TextView mName; // 存储文件夹名称的文本视图

        // 构造函数，接受上下文作为参数
        public FolderListItem(Context context) {
            super(context); // 调用父类构造函数
            inflate(context, R.layout.folder_list_item, this); // 加载布局文件
            mName = (TextView) findViewById(R.id.tv_folder_name); // 初始化文件夹名称文本视图
        }

        // 绑定文件夹名称到文本视图
        public void bind(String name) {
            mName.setText(name); // 设置文本视图的文本为文件夹名称
        }
    }
package net.micode.notes.ui; // 定义包名

import android.content.Context; // 导入上下文类
import android.database.Cursor; // 导入游标类
import android.view.View; // 导入视图类
import android.view.ViewGroup; // 导入视图组类
import android.widget.CursorAdapter; // 导入游标适配器类
import android.widget.LinearLayout; // 导入线性布局类
import android.widget.TextView; // 导入文本视图类

import net.micode.notes.R; // 导入资源类
import net.micode.notes.data.Notes; // 导入笔记数据类
import net.micode.notes.data.Notes.NoteColumns; // 导入笔记列类

public class FoldersListAdapter extends CursorAdapter { // 定义文件夹列表适配器类，继承自 CursorAdapter
    public static final String [] PROJECTION = { // 定义需要查询的字段数组
        NoteColumns.ID, // 笔记 ID 列
        NoteColumns.SNIPPET // 笔记摘要列
    };

    public static final int ID_COLUMN   = 0; // ID 列索引
    public static final int NAME_COLUMN = 1; // 名称列索引

    // 构造函数，接受上下文和游标作为参数
    public FoldersListAdapter(Context context, Cursor c) {
        super(context, c); // 调用父类构造函数
        // TODO Auto-generated constructor stub
    }

    // 创建新的视图
    @Override
    public View newView(Context context, Cursor cursor, ViewGroup parent) {
        return new FolderListItem(context); // 返回新的文件夹列表项视图
    }

    // 绑定视图与数据
    @Override
    public void bindView(View view, Context context, Cursor cursor) {
        if (view instanceof FolderListItem) { // 如果视图是 FolderListItem 的实例
            // 获取文件夹名称，判断是否为根文件夹
            String folderName = (cursor.getLong(ID_COLUMN) == Notes.ID_ROOT_FOLDER) ? context
                    .getString(R.string.menu_move_parent_folder) : cursor.getString(NAME_COLUMN);
            ((FolderListItem) view).bind(folderName); // 绑定文件夹名称到视图
        }
    }

    // 根据位置获取文件夹名称
    public String getFolderName(Context context, int position) {
        Cursor cursor = (Cursor) getItem(position); // 获取指定位置的游标
        // 判断是否为根文件夹，返回相应的名称
        return (cursor.getLong(ID_COLUMN) == Notes.ID_ROOT_FOLDER) ? context
                .getString(R.string.menu_move_parent_folder) : cursor.getString(NAME_COLUMN);
    }

    // 内部类，表示文件夹列表项
    private class FolderListItem extends LinearLayout { // 继承自 LinearLayout
        private TextView mName; // 存储文件夹名称的文本视图

        // 构造函数，接受上下文作为参数
        public FolderListItem(Context context) {
            super(context); // 调用父类构造函数
            inflate(context, R.layout.folder_list_item, this); // 加载布局文件
            mName = (TextView) findViewById(R.id.tv_folder_name); // 初始化文件夹名称文本视图
        }

        // 绑定文件夹名称到文本视图
        public void bind(String name) {
            mName.setText(name); // 设置文本视图的文本为文件夹名称
        }
    }
