/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.ui; // 定义包名

import java.util.Calendar; // 导入日历类

import net.micode.notes.R; // 导入资源类
import net.micode.notes.ui.DateTimePicker; // 导入日期时间选择器类
import net.micode.notes.ui.DateTimePicker.OnDateTimeChangedListener; // 导入日期时间改变监听器接口

import android.app.AlertDialog; // 导入警告对话框类
import android.content.Context; // 导入上下文类
import android.content.DialogInterface; // 导入对话框接口
import android.content.DialogInterface.OnClickListener; // 导入点击监听器接口
import android.text.format.DateFormat; // 导入日期格式类
import android.text.format.DateUtils; // 导入日期工具类

public class DateTimePickerDialog extends AlertDialog implements OnClickListener { // 定义日期时间选择对话框类，继承自 AlertDialog，并实现 OnClickListener 接口

    private Calendar mDate = Calendar.getInstance(); // 初始化当前日期的日历对象
    private boolean mIs24HourView; // 存储当前是否为 24 小时制的标志
    private OnDateTimeSetListener mOnDateTimeSetListener; // 日期时间设置监听器
    private DateTimePicker mDateTimePicker; // 日期时间选择器实例

    // 定义日期时间设置监听器接口
    public interface OnDateTimeSetListener {
        void OnDateTimeSet(AlertDialog dialog, long date); // 日期时间设置方法签名
    }

    // 构造函数，接受上下文和日期参数
    public DateTimePickerDialog(Context context, long date) {
        super(context); // 调用父类构造函数
        mDateTimePicker = new DateTimePicker(context); // 创建日期时间选择器实例
        setView(mDateTimePicker); // 设置对话框视图为日期时间选择器
        mDateTimePicker.setOnDateTimeChangedListener(new OnDateTimeChangedListener() { // 设置日期时间改变监听器
            public void onDateTimeChanged(DateTimePicker view, int year, int month,
                    int dayOfMonth, int hourOfDay, int minute) { // 当日期时间改变时调用
                mDate.set(Calendar.YEAR, year); // 更新日历中的年份
                mDate.set(Calendar.MONTH, month); // 更新日历中的月份
                mDate.set(Calendar.DAY_OF_MONTH, dayOfMonth); // 更新日历中的日期
                mDate.set(Calendar.HOUR_OF_DAY, hourOfDay); // 更新日历中的小时
                mDate.set(Calendar.MINUTE, minute); // 更新日历中的分钟
                updateTitle(mDate.getTimeInMillis()); // 更新对话框标题
            }
        });
        mDate.setTimeInMillis(date); // 设置日历的时间为传入的日期
        mDate.set(Calendar.SECOND, 0); // 将秒设置为 0
        mDateTimePicker.setCurrentDate(mDate.getTimeInMillis()); // 设置日期时间选择器的当前日期
        setButton(context.getString(R.string.datetime_dialog_ok), this); // 设置确定按钮
        setButton2(context.getString(R.string.datetime_dialog_cancel), (OnClickListener)null); // 设置取消按钮
        set24HourView(DateFormat.is24HourFormat(this.getContext())); // 设置是否使用 24 小时制
        updateTitle(mDate.getTimeInMillis()); // 更新对话框标题
    }

    // 设置 24 小时制模式
    public void set24HourView(boolean is24HourView) {
        mIs24HourView = is24HourView; // 更新 24 小时制标志
    }

    // 设置日期时间设置监听器
    public void setOnDateTimeSetListener(OnDateTimeSetListener callBack) {
        mOnDateTimeSetListener = callBack; // 更新监听器
    }

    // 更新对话框标题
    private void updateTitle(long date) {
        int flag = // 设置日期时间格式标志
            DateUtils.FORMAT_SHOW_YEAR | // 显示年份
            DateUtils.FORMAT_SHOW_DATE | // 显示日期
            DateUtils.FORMAT_SHOW_TIME; // 显示时间
        flag |= mIs24HourView ? DateUtils.FORMAT_24HOUR : DateUtils.FORMAT_12HOUR; // 根据模式添加相应的标志
        setTitle(DateUtils.formatDateTime(this.getContext(), date, flag)); // 设置对话框标题为格式化的日期时间
    }

    // 点击事件处理
    public void onClick(DialogInterface arg0, int arg1) {
        if (mOnDateTimeSetListener != null) { // 如果有设置的监听器
            mOnDateTimeSetListener.OnDateTimeSet(this, mDate.getTimeInMillis()); // 调用监听器方法，传递当前日期
        }
    }
}