/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.ui; // 定义包名

import android.content.Context; // 导入Context类
import android.database.Cursor; // 导入Cursor类
import android.util.Log; // 导入Log类
import android.view.View; // 导入View类
import android.view.ViewGroup; // 导入ViewGroup类
import android.widget.CursorAdapter; // 导入CursorAdapter类

import net.micode.notes.data.Notes; // 导入Notes数据类

import java.util.Collection; // 导入Collection接口
import java.util.HashMap; // 导入HashMap类
import java.util.HashSet; // 导入HashSet类
import java.util.Iterator; // 导入Iterator接口

public class NotesListAdapter extends CursorAdapter { // 定义 NotesListAdapter 类，继承自 CursorAdapter
    private static final String TAG = "NotesListAdapter"; // 日志标签

    private Context mContext; // 上下文
    private HashMap<Integer, Boolean> mSelectedIndex; // 用于记录选择状态的哈希映射
    private int mNotesCount; // 笔记数量
    private boolean mChoiceMode; // 选择模式标志

    public static class AppWidgetAttribute { // 定义内部类 AppWidgetAttribute，用于存储小部件的属性
        public int widgetId; // 小部件ID
        public int widgetType; // 小部件类型
    };

    public NotesListAdapter(Context context) { // 构造函数
        super(context, null); // 调用父类构造函数
        mSelectedIndex = new HashMap<Integer, Boolean>(); // 初始化选择索引映射
        mContext = context; // 设置上下文
        mNotesCount = 0; // 初始化笔记数量为0
    }

    @Override
    public View newView(Context context, Cursor cursor, ViewGroup parent) { // 创建新视图的方法
        return new NotesListItem(context); // 返回新的NotesListItem实例
    }

    @Override
    public void bindView(View view, Context context, Cursor cursor) { // 绑定视图与数据的方法
        if (view instanceof NotesListItem) { // 检查视图类型
            NoteItemData itemData = new NoteItemData(context, cursor); // 创建NoteItemData对象
            ((NotesListItem) view).bind(context, itemData, mChoiceMode, // 绑定数据到视图
                    isSelectedItem(cursor.getPosition())); // 传递当前项是否被选中的状态
        }
    }

    public void setCheckedItem(final int position, final boolean checked) { // 设置某项的选择状态
        mSelectedIndex.put(position, checked); // 更新选择状态
        notifyDataSetChanged(); // 通知数据变化，更新视图
    }

    public boolean isInChoiceMode() { // 判断是否处于选择模式
        return mChoiceMode; // 返回选择模式状态
    }

    public void setChoiceMode(boolean mode) { // 设置选择模式
        mSelectedIndex.clear(); // 清除之前的选择状态
        mChoiceMode = mode; // 更新选择模式状态
    }

    public void selectAll(boolean checked) { // 全选或取消全选的方法
        Cursor cursor = getCursor(); // 获取当前游标
        for (int i = 0; i < getCount(); i++) { // 遍历所有项
            if (cursor.moveToPosition(i)) { // 移动游标到指定位置
                if (NoteItemData.getNoteType(cursor) == Notes.TYPE_NOTE) { // 如果项的类型为笔记
                    setCheckedItem(i, checked); // 设置选择状态
                }
            }
        }
    }

    public HashSet<Long> getSelectedItemIds() { // 获取选中项ID集合
        HashSet<Long> itemSet = new HashSet<Long>(); // 创建ID集合
        for (Integer position : mSelectedIndex.keySet()) { // 遍历选择索引
            if (mSelectedIndex.get(position) == true) { // 如果该项被选中
                Long id = getItemId(position); // 获取对应的ID
                if (id == Notes.ID_ROOT_FOLDER) { // 如果是根文件夹ID
                    Log.d(TAG, "Wrong item id, should not happen"); // 日志输出错误信息
                } else {
                    itemSet.add(id); // 添加ID到集合
                }
            }
        }

        return itemSet; // 返回ID集合
    }

    public HashSet<AppWidgetAttribute> getSelectedWidget() { // 获取选中小部件的方法
        HashSet<AppWidgetAttribute> itemSet = new HashSet<AppWidgetAttribute>(); // 创建小部件属性集合
        for (Integer position : mSelectedIndex.keySet()) { // 遍历选择索引
            if (mSelectedIndex.get(position) == true) { // 如果该项被选中
                Cursor c = (Cursor) getItem(position); // 获取相应游标
                if (c != null) { // 如果游标不为空
                    AppWidgetAttribute widget = new AppWidgetAttribute(); // 创建小部件属性对象
                    NoteItemData item = new NoteItemData(mContext, c); // 创建笔记数据项
                    widget.widgetId = item.getWidgetId(); // 获取小部件ID
                    widget.widgetType = item.getWidgetType(); // 获取小部件类型
                    itemSet.add(widget); // 将小部件属性添加到集合
                    // 不要在这里关闭游标，只有适配器可以关闭它
                } else {
                    Log.e(TAG, "Invalid cursor"); // 日志输出错误信息
                    return null; // 返回null
                }
            }
        }
        return itemSet; // 返回小部件属性集合
    }

    public int getSelectedCount() { // 获取选中项数量的方法
        Collection<Boolean> values = mSelectedIndex.values(); // 获取所有选择状态
        if (null == values) { // 如果值为空
            return 0; // 返回0
        }
        Iterator<Boolean> iter = values.iterator(); // 创建迭代器
        int count = 0; // 初始化计数器
        while (iter.hasNext()) { // 遍历选择状态
            if (true == iter.next()) { // 如果选中
                count++; // 增加计数
            }
        }
        return count; // 返回选中数量
    }

    public boolean isAllSelected() { // 判断是否全部选中
        int checkedCount = getSelectedCount(); // 获取已选中项数量
        return (checkedCount != 0 && checkedCount == mNotesCount); // 如果已选数量不为0并且与总数量相等，返回true
    }

    public boolean isSelectedItem(final int position) { // 判断指定项是否被选中
        if (null == mSelectedIndex.get(position)) { // 如果没有该项的选择状态
            return false; // 返回false
        }
        return mSelectedIndex.get(position); // 返回该项的选择状态
    }

    @Override
    protected void onContentChanged() { // 当内容发生变化时调用
        super.onContentChanged(); // 调用父类方法
        calcNotesCount(); // 计算笔记数量
    }

    @Override
    public void changeCursor(Cursor cursor) { // 更改游标的方法
        super.changeCursor(cursor); // 调用父类方法
        calcNotesCount(); // 计算笔记数量
    }

    private void calcNotesCount() { // 计算笔记数量的方法
        mNotesCount = 0; // 初始化数量
        for (int i = 0; i < getCount(); i++) { // 遍历所有项目
            Cursor c = (Cursor) getItem(i); // 获取当前项目的游标
            if (c != null) { // 如果游标不为空
                if (NoteItemData.getNoteType(c) == Notes.TYPE_NOTE) { // 如果游标类型为笔记
                    mNotesCount++; // 增加计数
                }
            } else {
                Log.e(TAG, "Invalid cursor"); // 日志输出错误信息
                return; // 返回
            }
        }
    }
}