/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.widget;

import android.appwidget.AppWidgetManager; // 导入AppWidgetManager类，用于管理小部件
import android.appwidget.AppWidgetProvider; // 导入AppWidgetProvider类，用于小部件提供者
import android.content.ComponentName; // 导入ComponentName类，用于组件名称
import android.content.Context; // 导入Context类，用于获取应用程序环境信息
import android.content.Intent; // 导入Intent类，用于传递消息
import android.content.SharedPreferences; // 导入SharedPreferences类，用于存储设置
import android.graphics.Bitmap; // 导入Bitmap类，用于图像操作
import android.graphics.BitmapFactory; // 导入BitmapFactory类，用于图像解码
import android.os.Bundle; // 导入Bundle类，用于传递数据
import android.util.Log; // 导入Log类，用于日志记录
import android.view.View; // 导入View类，用于视图操作
import android.widget.RemoteViews; // 导入RemoteViews类，用于远程视图操作
import android.widget.RemoteViewsService; // 导入RemoteViewsService类，用于服务小部件
import android.widget.Toast; // 导入Toast类，用于显示提示信息

import net.micode.notes.R; // 导入R类，用于访问应用程序资源
import net.micode.notes.data.Notes; // 导入Notes类，用于访问笔记数据
import net.micode.notes.tool.ResourceParser; // 导入ResourceParser类，用于解析资源

import java.util.ArrayList; // 导入ArrayList类，用于存储数据
import java.util.List; // 导入List类，用于数据集合

/**
 * NoteWidgetProvider_2x类，继承自AppWidgetProvider，用于2x大小的小部件
 */
public class NoteWidgetProvider_2x extends AppWidgetProvider {

    private static final String TAG = "NoteWidgetProvider_2x"; // 日志标签
    private static final String WIDGET_PREFS = "widget_prefs"; // 小部件设置的SharedPreferences名称
    private static final String WIDGET_BG_COLOR_KEY = "widget_bg_color"; // 小部件背景颜色的键
    private static final String WIDGET_FONT_SIZE_KEY = "widget_font_size"; // 小部件字体大小的键
    private static final String WIDGET_SYNC_ENABLED_KEY = "widget_sync_enabled"; // 小部件数据同步是否启用的键
    private static final String WIDGET_SYNC_URL_KEY = "widget_sync_url"; // 小部件数据同步的URL的键

    /**
     * 当小部件需要更新时调用
     * @param context 应用程序上下文
     * @param appWidgetManager 小部件管理器
     * @param appWidgetIds 需要更新的小部件ID数组
     */
    @Override
    public void onUpdate(Context context, AppWidgetManager appWidgetManager, int[] appWidgetIds) {
        Log.d(TAG, "onUpdate called");

        // 获取远程视图对象
        RemoteViews views = new RemoteViews(context.getPackageName(), getLayoutId());

        // 设置点击事件
        Intent clickIntent = new Intent(context, NoteWidgetProvider_2x.class);
        clickIntent.setAction(AppWidgetManager.ACTION_APPWIDGET_UPDATE);
        clickIntent.putExtra(AppWidgetManager.EXTRA_APPWIDGET_IDS, appWidgetIds);
        PendingIntent pendingIntent = PendingIntent.getBroadcast(context, 0, clickIntent, PendingIntent.FLAG_UPDATE_CURRENT);
        views.setOnClickPendingIntent(R.id.widget_2x_click_area, pendingIntent);

        // 设置列表适配器
        Intent intent = new Intent(context, NoteWidgetService.class);
        views.setRemoteAdapter(R.id.widget_2x_list_view, intent);

        // 更新小部件视图
        for (int appWidgetId : appWidgetIds) {
            appWidgetManager.updateAppWidget(appWidgetId, views);
        }

        // 检查数据同步是否启用
        SharedPreferences prefs = context.getSharedPreferences(WIDGET_PREFS, Context.MODE_PRIVATE);
        boolean syncEnabled = prefs.getBoolean(WIDGET_SYNC_ENABLED_KEY, false);
        String syncUrl = prefs.getString(WIDGET_SYNC_URL_KEY, "");

        if (syncEnabled && !syncUrl.isEmpty()) {
            // 启动数据同步服务
            Intent syncIntent = new Intent(context, NoteSyncService.class);
            syncIntent.putExtra("sync_url", syncUrl);
            context.startService(syncIntent);
        }
    }

    /**
     * 获取小部件的布局ID
     * @return 小部件的布局资源ID
     */
    @Override
    protected int getLayoutId() {
        // 返回2x小部件的布局资源ID
        return R.layout.widget_2x;
    }

    /**
     * 获取小部件的背景资源ID
     * @param bgId 背景ID
     * @return 小部件的背景资源ID
     */
    @Override
    protected int getBgResourceId(int bgId) {
        // 调用ResourceParser类的getWidget2xBgResource方法，获取2x小部件的背景资源ID
        return ResourceParser.WidgetBgResources.getWidget2xBgResource(bgId);
    }

    /**
     * 获取小部件类型
     * @return 小部件类型
     */
    @Override
    protected int getWidgetType() {
        // 返回2x小部件类型
        return Notes.TYPE_WIDGET_2X;
    }

    /**
     * 当小部件被删除时调用
     * @param context 应用程序上下文
     * @param appWidgetIds 被删除的小部件ID数组
     */
    @Override
    public void onDeleted(Context context, int[] appWidgetIds) {
        Log.d(TAG, "onDeleted called");

        // 调用父类的onDeleted方法
        super.onDeleted(context, appWidgetIds);

        // 执行删除小部件时需要的操作，例如清除数据等
        Notes.deleteWidgetData(context, appWidgetIds);

        // 显示提示信息
        Toast.makeText(context, "小部件已删除", Toast.LENGTH_SHORT).show();
    }

    /**
     * 当小部件被启用时调用
     * @param context 应用程序上下文
     */
    @Override
    public void onEnabled(Context context) {
        Log.d(TAG, "onEnabled called");

        // 调用父类的onEnabled方法
        super.onEnabled(context);

        // 执行小部件启用时需要的操作，例如初始化数据等
        Notes.initWidgetSettings(context);

        // 显示提示信息
        Toast.makeText(context, "小部件已启用", Toast.LENGTH_SHORT).show();
    }

    /**
     * 当小部件被禁用时调用
     * @param context 应用程序上下文
     */
    @Override
    public void onDisabled(Context context) {
        Log.d(TAG, "onDisabled called");

        // 调用父类的onDisabled方法
        super.onDisabled(context);

        // 执行小部件禁用时需要的操作，例如保存数据等
        Notes.saveWidgetSettings(context);

        // 显示提示信息
        Toast.makeText(context, "小部件已禁用", Toast.LENGTH_SHORT).show();
    }

    /**
     * 当小部件大小发生变化时调用
     * @param context 应用程序上下文
     * @param appWidgetManager 小部件管理器
     * @param appWidgetId 小部件ID
     * @param newOptions 小部件的新选项
     */
    @Override
    public void onAppWidgetOptionsChanged(Context context, AppWidgetManager appWidgetManager, int appWidgetId, AppWidgetManager.AppWidgetOptions newOptions) {
        Log.d(TAG, "onAppWidgetOptionsChanged called");

        // 获取新的宽度和高度
        int width = newOptions.minWidth;
        int height = newOptions.minHeight;

        // 根据新的大小调整小部件的布局
        if (width > height) {
            // 如果宽度大于高度，调整为横向布局
            RemoteViews views = new RemoteViews(context.getPackageName(), R.layout.widget_2x_horizontal);
            appWidgetManager.updateAppWidget(appWidgetId, views);
        } else {
            // 如果高度大于宽度，调整为纵向布局
            RemoteViews views = new RemoteViews(context.getPackageName(), R.layout.widget_2x_vertical);
            appWidgetManager.updateAppWidget(appWidgetId, views);
        }

        // 显示提示信息
        Toast.makeText(context, "小部件大小已调整", Toast.LENGTH_SHORT).show();
    }

    /**
     * 当小部件接收到广播时调用
     * @param context 应用程序上下文
     * @param intent 接收到的Intent
     */
    @Override
    public void onReceive(Context context, Intent intent) {
        Log.d(TAG, "onReceive called");

        // 调用父类的onReceive方法
        super.onReceive(context, intent);

        String action = intent.getAction();
        if (AppWidgetManager.ACTION_APPWIDGET_UPDATE.equals(action)) {
            // 更新小部件
            int[] appWidgetIds = intent.getIntArrayExtra(AppWidgetManager.EXTRA_APPWIDGET_IDS);
            onUpdate(context, AppWidgetManager.getInstance(context), appWidgetIds);
        } else if ("com.example.notes.widget.action.EDIT_NOTE".equals(action)) {
            // 编辑笔记
            int appWidgetId = intent.getIntExtra("app_widget_id", -1);
            int noteId = intent.getIntExtra("note_id", -1);
            editNote(context, appWidgetId, noteId);
        } else if ("com.example.notes.widget.action.NOTE_SYNCED".equals(action)) {
            // 笔记数据同步完成
            int appWidgetId = intent.getIntExtra("app_widget_id", -1);
            Bundle extras = intent.getExtras();
            if (extras != null) {
                List<Notes.Note> syncedNotes = extras.getParcelableArrayList("synced_notes");
                if (syncedNotes != null) {
                    updateNotes(context, appWidgetId, syncedNotes);
                }
            }
        }
    }

    /**
     * 编辑笔记
     * @param context 应用程序上下文
     * @param appWidgetId 小部件ID
     * @param noteId 笔记ID
     */
    private void editNote(Context context, int appWidgetId, int noteId) {
        Log.d(TAG, "editNote called");

        // 获取笔记数据
        Notes.Note note = Notes.getNoteById(context, noteId);
        if (note != null) {
            // 显示编辑对话框
            showEditDialog(context, appWidgetId, note);
        } else {
            // 显示错误提示信息
            Toast.makeText(context, "笔记不存在", Toast.LENGTH_SHORT).show();
        }
    }

    /**
     * 显示编辑对话框
     * @param context 应用程序上下文
     * @param appWidgetId 小部件ID
     * @param note 笔记数据
     */
    private void showEditDialog(Context context, int appWidgetId, Notes.Note note) {
        Log.d(TAG, "showEditDialog called");

        // 创建编辑对话框
        AlertDialog.Builder builder = new AlertDialog.Builder(context);
        builder.setTitle("编辑笔记");
        View dialogView = LayoutInflater.from(context).inflate(R.layout.dialog_edit_note, null);
        EditText editText = dialogView.findViewById(R.id.edit_text);
        editText.setText(note.getContent());

        builder.setView(dialogView);
        builder.setPositiveButton("保存", new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                String newContent = editText.getText().toString();
                if (!newContent.isEmpty()) {
                    // 更新笔记数据
                    note.setContent(newContent);
                    Notes.updateNote(context, note);

                    // 更新小部件视图
                    updateAppWidget(context, appWidgetId);
                } else {
                    // 显示错误提示信息
                    Toast.makeText(context, "笔记内容不能为空", Toast.LENGTH_SHORT).show();
                }
            }
        });
        builder.setNegativeButton("取消", null);

        // 显示对话框
        builder.show();
    }

    /**
     * 更新小部件视图
     * @param context 应用程序上下文
     * @param appWidgetId 小部件ID
     */
    private void updateAppWidget(Context context, int appWidgetId) {
        Log.d(TAG, "updateAppWidget called");

        // 获取远程视图对象
        RemoteViews views = new RemoteViews(context.getPackageName(), getLayoutId());

        // 设置列表适配器
        Intent intent = new Intent(context, NoteWidgetService.class);
        views.setRemoteAdapter(R.id.widget_2x_list_view, intent);

        // 更新小部件视图
        AppWidgetManager appWidgetManager = AppWidgetManager.getInstance(context);
        appWidgetManager.updateAppWidget(appWidgetId, views);
    }

    /**
     * 更新笔记数据
     * @param context 应用程序上下文
     * @param appWidgetId 小部件ID
     * @param syncedNotes 同步的笔记数据列表
     */
    private void updateNotes(Context context, int appWidgetId, List<Notes.Note> syncedNotes) {
        Log.d(TAG, "updateNotes called");

        // 更新笔记数据
        Notes.updateNotes(context, syncedNotes);

        // 更新小部件视图
        updateAppWidget(context, appWidgetId);
    }

    /**
     * 获取小部件的背景颜色
     * @param context 应用程序上下文
     * @return 小部件的背景颜色
     */
    private int getWidgetBgColor(Context context) {
        SharedPreferences prefs = context.getSharedPreferences(WIDGET_PREFS, Context.MODE_PRIVATE);
        return prefs.getInt(WIDGET_BG_COLOR_KEY, 0xFFE0E0E0); // 默认背景颜色为浅灰色
    }

    /**
     * 获取小部件的字体大小
     * @param context 应用程序上下文
     * @return 小部件的字体大小
     */
    private int getWidgetFontSize(Context context) {
        SharedPreferences prefs = context.getSharedPreferences(WIDGET_PREFS, Context.MODE_PRIVATE);
        return prefs.getInt(WIDGET_FONT_SIZE_KEY, 16); // 默认字体大小为16sp
    }

    /**
     * 设置小部件的背景颜色
     * @param context 应用程序上下文
     * @param color 小部件的背景颜色
     */
    private void setWidgetBgColor(Context context, int color) {
        SharedPreferences prefs = context.getSharedPreferences(WIDGET_PREFS, Context.MODE_PRIVATE);
        prefs.edit().putInt(WIDGET_BG_COLOR_KEY, color).apply();
    }

    /**
     * 设置小部件的字体大小
     * @param context 应用程序上下文
     * @param size 小部件的字体大小
     */
    private void setWidgetFontSize(Context context, int size) {
        SharedPreferences prefs = context.getSharedPreferences(WIDGET_PREFS, Context.MODE_PRIVATE);
        prefs.edit().putInt(WIDGET_FONT_SIZE_KEY, size).apply();
    }

    /**
     * 启用或禁用数据同步
     * @param context 应用程序上下文
     * @param enabled 是否启用数据同步
     */
    private void setSyncEnabled(Context context, boolean enabled) {
        SharedPreferences prefs = context.getSharedPreferences(WIDGET_PREFS, Context.MODE_PRIVATE);
        prefs.edit().putBoolean(WIDGET_SYNC_ENABLED_KEY, enabled).apply();
    }

    /**
     * 设置数据同步的URL
     * @param context 应用程序上下文
     * @param url 数据同步的URL
     */
    private void setSyncUrl(Context context, String url) {
        SharedPreferences prefs = context.getSharedPreferences(WIDGET_PREFS, Context.MODE_PRIVATE);
        prefs.edit().putString(WIDGET_SYNC_URL_KEY, url).apply();
    }

    /**
     * 获取小部件的配置活动
     * @param context 应用程序上下文
     * @return 小部件的配置活动
     */
    public static Intent getConfigActivityIntent(Context context) {
        Intent intent = new Intent(context, NoteWidgetConfigureActivity.class);
        intent.putExtra(AppWidgetManager.EXTRA_APPWIDGET_ID, -1); // 默认值为-1，表示未配置
        return intent;
    }

    /**
     * 获取小部件的配置活动结果
     * @param context 应用程序上下文
     * @param resultIntent 配置活动的结果Intent
     * @return 配置结果
     */
    public static boolean getConfigActivityResult(Context context, Intent resultIntent) {
        if (resultIntent == null) {
            return false;
        }

        int appWidgetId = resultIntent.getIntExtra(AppWidgetManager.EXTRA_APPWIDGET_ID, -1);
        if (appWidgetId == -1) {
            return false;
        }

        // 获取配置结果
        boolean result = resultIntent.getBooleanExtra("config_result", false);
        if (result) {
            // 更新小部件
            AppWidgetManager appWidgetManager = AppWidgetManager.getInstance(context);
            RemoteViews views = new RemoteViews(context.getPackageName(), getLayoutId());
            appWidgetManager.updateAppWidget(appWidgetId, views);
        }

        return result;
    }

    /**
     * 获取小部件的背景图像
     * @param context 应用程序上下文
     * @param bgId 背景ID
     * @return 小部件的背景图像
     */
    private Bitmap getWidgetBgImage(Context context, int bgId) {
        // 根据背景ID获取背景图像资源ID
        int bgResId = ResourceParser.WidgetBgResources.getWidget2xBgResource(bgId);

        // 解码背景图像资源
        BitmapFactory.Options options = new BitmapFactory.Options();
        options.inSampleSize = 2; // 缩小图像以减少内存使用
        return BitmapFactory.decodeResource(context.getResources(), bgResId, options);
    }

    /**
     * 设置小部件的背景图像
     * @param context 应用程序上下文
     * @param appWidgetId 小部件ID
     * @param bgImage 小部件的背景图像
     */
    private void setWidgetBgImage(Context context, int appWidgetId, Bitmap bgImage) {
        // 获取远程视图对象
        RemoteViews views = new RemoteViews(context.getPackageName(), getLayoutId());

        // 设置背景图像
        views.setImageViewBitmap(R.id.widget_2x_background, bgImage);

        // 更新小部件视图
        AppWidgetManager appWidgetManager = AppWidgetManager.getInstance(context);
        appWidgetManager.updateAppWidget(appWidgetId, views);
    }

    /**
     * 获取小部件的背景资源ID
     * @param context 应用程序上下文
     * @param bgId 背景ID
     * @return 小部件的背景资源ID
     */
    private int getWidgetBgResourceId(Context context, int bgId) {
        // 调用ResourceParser类的getWidget2xBgResource方法，获取2x小部件的背景资源ID
        return ResourceParser.WidgetBgResources.getWidget2xBgResource(bgId);
    }

    /**
     * 设置小部件的背景资源ID
     * @param context 应用程序上下文
     * @param appWidgetId 小部件ID
     * @param bgResId 小部件的背景资源ID
     */
    private void setWidgetBgResourceId(Context context, int appWidgetId, int bgResId) {
        // 获取远程视图对象
        RemoteViews views = new RemoteViews(context.getPackageName(), getLayoutId());

        // 设置背景资源ID
        views.setImageViewResource(R.id.widget_2x_background, bgResId);

        // 更新小部件视图
        AppWidgetManager appWidgetManager = AppWidgetManager.getInstance(context);
        appWidgetManager.updateAppWidget(appWidgetId, views);
    }

    /**
     * 获取小部件的类型
     * @return 小部件类型
     */
    private int getWidgetType() {
        // 返回2x小部件类型
        return Notes.TYPE_WIDGET_2X;
    }

    /**
     * 设置小部件的类型
     * @param context 应用程序上下文
     * @param appWidgetId 小部件ID
     * @param widgetType 小部件类型
     */
    private void setWidgetType(Context context, int appWidgetId, int widgetType) {
        // 获取远程视图对象
        RemoteViews views = new RemoteViews(context.getPackageName(), getLayoutId());

        // 根据小部件类型设置不同的布局
        switch (widgetType) {
            case Notes.TYPE_WIDGET_2X:
                views.setViewVisibility(R.id.widget_2x_list_view, View.VISIBLE);
                views.setViewVisibility(R.id.widget_2x_text_view, View.GONE);
                break;
            case Notes.TYPE_WIDGET_1X:
                views.setViewVisibility(R.id.widget_2x_list_view, View.GONE);
                views.setViewVisibility(R.id.widget_2x_text_view, View.VISIBLE);
                break;
            default:
                // 默认显示列表视图
                views.setViewVisibility(R.id.widget_2x_list_view, View.VISIBLE);
                views.setViewVisibility(R.id.widget_2x_text_view, View.GONE);
                break;
        }

        // 更新小部件视图
        AppWidgetManager appWidgetManager = AppWidgetManager.getInstance(context);
        appWidgetManager.updateAppWidget(appWidgetId, views);
    }

    /**
     * 获取小部件的动画效果
     * @param context 应用程序上下文
     * @return 小部件的动画效果
     */
    private Animation getWidgetAnimation(Context context) {
        // 创建动画效果
        Animation animation = new AlphaAnimation(0.0f, 1.0f);
        animation.setDuration(1000); // 动画持续时间为1秒
        animation.setInterpolator(new AccelerateDecelerateInterpolator());
        return animation;
    }

    /**
     * 设置小部件的动画效果
     * @param context 应用程序上下文
     * @param appWidgetId 小部件ID
     * @param animation 小部件的动画效果
     */
    private void setWidgetAnimation(Context context, int appWidgetId, Animation animation) {
        // 获取远程视图对象
        RemoteViews views = new RemoteViews(context.getPackageName(), getLayoutId());

        // 设置动画效果
        views.setAnimation(R.id.widget_2x_list_view, animation);

        // 更新小部件视图
        AppWidgetManager appWidgetManager = AppWidgetManager.getInstance(context);
        appWidgetManager.updateAppWidget(appWidgetId, views);
    }

    /**
     * 获取小部件的滑动手势
     * @param context 应用程序上下文
     * @return 小部件的滑动手势
     */
    private GestureDetector getWidgetGestureDetector(Context context) {
        // 创建滑动手势检测器
        GestureDetector gestureDetector = new GestureDetector(context, new GestureDetector.SimpleOnGestureListener() {
            @Override
            public boolean onFling(MotionEvent e1, MotionEvent e2, float velocityX, float velocityY) {
                // 检测滑动手势
                if (Math.abs(velocityX) > Math.abs(velocityY)) {
                    if (velocityX > 0) {
                        // 向右滑动
                        onSwipeRight(context);
                    } else {
                        // 向左滑动
                        onSwipeLeft(context);
                    }
                }
                return true;
            }
        });
        return gestureDetector;
    }

    /**
     * 向右滑动时的处理
     * @param context 应用程序上下文
     */
    private void onSwipeRight(Context context) {
        Log.d(TAG, "onSwipeRight called");

        // 向右滑动时的处理逻辑
        // 例如，切换到上一条笔记
        int appWidgetId = AppWidgetManager.getInstance(context).getAppWidgetIds(new ComponentName(context, NoteWidgetProvider_2x.class))[0];
        Notes.Note note = Notes.getPreviousNote(context, appWidgetId);
        if (note != null) {
            updateAppWidget(context, appWidgetId);
        } else {
            Toast.makeText(context, "已经是第一条笔记", Toast.LENGTH_SHORT).show();
        }
    }

    /**
     * 向左滑动时的处理
     * @param context 应用程序上下文
     */
    private void onSwipeLeft(Context context) {
        Log.d(TAG, "onSwipeLeft called");

        // 向左滑动时的处理逻辑
        // 例如，切换到下一条笔记
        int appWidgetId = AppWidgetManager.getInstance(context).getAppWidgetIds(new ComponentName(context, NoteWidgetProvider_2x.class))[0];
        Notes.Note note = Notes.getNextNote(context, appWidgetId);
        if (note != null) {
            updateAppWidget(context, appWidgetId);
        } else {
            Toast.makeText(context, "已经是最后一条笔记", Toast.LENGTH_SHORT).show();
        }
    }

    /**
     * 获取小部件的点击事件监听器
     * @param context 应用程序上下文
     * @return 小部件的点击事件监听器
     */
    private View.OnClickListener getWidgetClickListener(Context context) {
        return new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                // 处理小部件的点击事件
                // 例如，打开笔记编辑页面
                Intent intent = new Intent(context, NoteEditActivity.class);
                intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
                context.startActivity(intent);
            }
        };
    }

    /**
     * 设置小部件的点击事件监听器
     * @param context 应用程序上下文
     * @param appWidgetId 小部件ID
     * @param clickListener 小部件的点击事件监听器
     */
    private void setWidgetClickListener(Context context, int appWidgetId, View.OnClickListener clickListener) {
        // 获取远程视图对象
        RemoteViews views = new RemoteViews(context.getPackageName(), getLayoutId());

        // 设置点击事件监听器
        views.setOnClickPendingIntent(R.id.widget_2x_click_area, PendingIntent.getBroadcast(context, 0, new Intent(), 0));

        // 更新小部件视图
        AppWidgetManager appWidgetManager = AppWidgetManager.getInstance(context);
        appWidgetManager.updateAppWidget(appWidgetId, views);
    }

    /**
     * 获取小部件的长按事件监听器
     * @param context 应用程序上下文
     * @return 小部件的长按事件监听器
     */
    private View.OnLongClickListener getWidgetLongClickListener(Context context) {
        return new View.OnLongClickListener() {
            @Override
            public boolean onLongClick(View v) {
                // 处理小部件的长按事件
                // 例如，显示快捷菜单
                showShortcutMenu(context);
                return true;
            }
        };
    }

    /**
     * 显示快捷菜单
     * @param context 应用程序上下文
     */
    private void showShortcutMenu(Context context) {
        // 创建快捷菜单
        PopupMenu popupMenu = new PopupMenu(context, null);
        popupMenu.getMenuInflater().inflate(R.menu.widget_shortcut_menu, popupMenu.getMenu());
        popupMenu.setOnMenuItemClickListener(new PopupMenu.OnMenuItemClickListener() {
            @Override
            public boolean onMenuItemClick(MenuItem item) {
                switch (item.getItemId()) {
                    case R.id.menu_edit_note:
                        // 编辑笔记
                        editNote(context, AppWidgetManager.getInstance(context).getAppWidgetIds(new ComponentName(context, NoteWidgetProvider_2x.class))[0], Notes.getCurrentNoteId(context));
                        break;
                    case R.id.menu_delete_note:
                        // 删除笔记
                        deleteNote(context, Notes.getCurrentNoteId(context));
                        break;
                    case R.id.menu_sync_notes:
                        // 同步笔记
                        syncNotes(context);
                        break;
                    default:
                        break;
                }
                return true;
            }
        });

        // 显示快捷菜单
        popupMenu.show();
    }

    /**
     * 删除笔记
     * @param context 应用程序上下文
     * @param noteId 笔记ID
     */
    private void deleteNote(Context context, int noteId) {
        // 删除笔记数据
        Notes.deleteNote(context, noteId);

        // 更新小部件视图
        int appWidgetId = AppWidgetManager.getInstance(context).getAppWidgetIds(new ComponentName(context, NoteWidgetProvider_2x.class))[0];
        updateAppWidget(context, appWidgetId);

        // 显示提示信息
        Toast.makeText(context, "笔记已删除", Toast.LENGTH_SHORT).show();
    }

    /**
     * 同步笔记
     * @param context 应用程序上下文
     */
    private void syncNotes(Context context) {
        // 获取数据同步的URL
        SharedPreferences prefs = context.getSharedPreferences(WIDGET_PREFS, Context.MODE_PRIVATE);
        String syncUrl = prefs.getString(WIDGET_SYNC_URL_KEY, "");

        if (!syncUrl.isEmpty()) {
            // 启动数据同步服务
            Intent syncIntent = new Intent(context, NoteSyncService.class);
            syncIntent.putExtra("sync_url", syncUrl);
            context.startService(syncIntent);
        } else {
            // 显示错误提示信息
            Toast.makeText(context, "请设置数据同步的URL", Toast.LENGTH_SHORT).show();
        }
    }

    /**
     * 获取小部件的配置信息
     * @param context 应用程序上下文
     * @return 小部件的配置信息
     */
    private String getWidgetConfigInfo(Context context) {
        // 获取小部件的配置信息
        SharedPreferences prefs = context.getSharedPreferences(WIDGET_PREFS, Context.MODE_PRIVATE);
        String configInfo = "背景颜色: " + prefs.getInt(WIDGET_BG_COLOR_KEY, 0xFFE0E0E0) + "\n";
        configInfo += "字体大小: " + prefs.getInt(WIDGET_FONT_SIZE_KEY, 16) + "sp\n";
        configInfo += "数据同步: " + (prefs.getBoolean(WIDGET_SYNC_ENABLED_KEY, false) ? "启用" : "禁用") + "\n";
        configInfo += "同步URL: " + prefs.getString(WIDGET_SYNC_URL_KEY, "");
        return configInfo;
    }

    /**
     * 设置小部件的配置信息
     * @param context 应用程序上下文
     * @param configInfo 小部件的配置信息
     */
    private void setWidgetConfigInfo(Context context, String configInfo) {
        // 解析配置信息
        String[] configItems = configInfo.split("\n");
        for (String item : configItems) {
            String[] keyValue = item.split(": ");
            if (keyValue.length == 2) {
                String key = keyValue[0];
                String value = keyValue[1];
                if ("背景颜色".equals(key)) {
                    setWidgetBgColor(context, Integer.parseInt(value));
                } else if ("字体大小".equals(key)) {
                    setWidgetFontSize(context, Integer.parseInt(value));
                } else if ("数据同步".equals(key)) {
                    setSyncEnabled(context, "启用".equals(value));
                } else if ("同步URL".equals(key)) {
                    setSyncUrl(context, value);
                }
            }
        }
    }

    /**
     * 获取小部件的笔记列表
     * @param context 应用程序上下文
     * @return 小部件的笔记列表
     */
    private List<Notes.Note> getWidgetNotes(Context context) {
        // 获取笔记数据
        return Notes.getAllNotes(context);
    }

    /**
     * 设置小部件的笔记列表
     * @param context 应用程序上下文
     * @param notes 小部件的笔记列表
     */
    private void setWidgetNotes(Context context, List<Notes.Note> notes) {
        // 更新笔记数据
        Notes.updateNotes(context, notes);

        // 更新小部件视图
        int appWidgetId = AppWidgetManager.getInstance(context).getAppWidgetIds(new ComponentName(context, NoteWidgetProvider_2x.class))[0];
        updateAppWidget(context, appWidgetId);
    }

    /**
     * 获取小部件的当前笔记
     * @param context 应用程序上下文
     * @return 小部件的当前笔记
     */
    private Notes.Note getWidgetCurrentNote(Context context) {
        // 获取当前笔记数据
        return Notes.getCurrentNote(context);
    }

    /**
     * 设置小部件的当前笔记
     * @param context 应用程序上下文
     * @param note 小部件的当前笔记
     */
    private void setWidgetCurrentNote(Context context, Notes.Note note) {
        // 更新当前笔记数据
        Notes.setCurrentNote(context, note);

        // 更新小部件视图
        int appWidgetId = AppWidgetManager.getInstance(context).getAppWidgetIds(new ComponentName(context, NoteWidgetProvider_2x.class))[0];
        updateAppWidget(context, appWidgetId);
    }

    /**
     * 获取小部件的笔记编辑状态
     * @param context 应用程序上下文
     * @return 小部件的笔记编辑状态
     */
    private boolean getWidgetEditState(Context context) {
        // 获取笔记编辑状态
        return Notes.isNoteEditing(context);
    }

    /**
     * 设置小部件的笔记编辑状态
     * @param context 应用程序上下文
     * @param editState 小部件的笔记编辑状态
     */
    private void setWidgetEditState(Context context, boolean editState) {
        // 更新笔记编辑状态
        Notes.setNoteEditing(context, editState);

        // 更新小部件视图
        int appWidgetId = AppWidgetManager.getInstance(context).getAppWidgetIds(new ComponentName(context, NoteWidgetProvider_2x.class))[0];
        updateAppWidget(context, appWidgetId);
    }

    /**
     * 获取小部件的笔记同步状态
     * @param context 应用程序上下文
     * @return 小部件的笔记同步状态
     */
    private boolean getWidgetSyncState(Context context) {
        // 获取笔记同步状态
        return Notes.isNoteSyncing(context);
    }

    /**
     * 设置小部件的笔记同步状态
     * @param context 应用程序上下文
     * @param syncState 小部件的笔记同步状态
     */
    private void setWidgetSyncState(Context context, boolean syncState) {
        // 更新笔记同步状态
        Notes.setNoteSyncing(context, syncState);

        // 更新小部件视图
        int appWidgetId = AppWidgetManager.getInstance(context).getAppWidgetIds(new ComponentName(context, NoteWidgetProvider_2x.class))[0];
        updateAppWidget(context, appWidgetId);
    }

    /**
     * 获取小部件的笔记数据同步服务
     * @param context 应用程序上下文
     * @return 小部件的笔记数据同步服务
     */
    private Intent getWidgetSyncServiceIntent(Context context) {
        // 创建笔记数据同步服务的Intent
        Intent syncIntent = new Intent(context, NoteSyncService.class);
        return syncIntent;
    }

    /**
     * 启动小部件的笔记数据同步服务
     * @param context 应用程序上下文
     */
    private void startWidgetSyncService(Context context) {
        // 启动笔记数据同步服务
        Intent syncIntent = getWidgetSyncServiceIntent(context);
        context.startService(syncIntent);
    }

    /**
     * 停止小部件的笔记数据同步服务
     * @param context 应用程序上下文
     */
    private void stopWidgetSyncService(Context context) {
        // 停止笔记数据同步服务
        Intent syncIntent = getWidgetSyncServiceIntent(context);
        context.stopService(syncIntent);
    }

    /**
     * 获取小部件的笔记数据同步状态监听器
     * @param context 应用程序上下文
     * @return 小部件的笔记数据同步状态监听器
     */
    private BroadcastReceiver getWidgetSyncStateReceiver(Context context) {
        return new BroadcastReceiver() {
            @Override
            public void onReceive(Context context, Intent intent) {
                // 处理笔记数据同步状态变化
                boolean syncState = intent.getBooleanExtra("sync_state", false);
                setWidgetSyncState(context, syncState);
            }
        };
    }

    /**
     * 注册小部件的笔记数据同步状态监听器
     * @param context 应用程序上下文
     */
    private void registerWidgetSyncStateReceiver(Context context) {
        // 注册笔记数据同步状态监听器
        IntentFilter filter = new IntentFilter("com.example.notes.widget.action.NOTE_SYNC_STATE_CHANGED");
        context.registerReceiver(getWidgetSyncStateReceiver(context), filter);
    }

    /**
     * 注销小部件的笔记数据同步状态监听器
     * @param context 应用程序上下文
     */
    private void unregisterWidgetSyncStateReceiver(Context context) {
        // 注销笔记数据同步状态监听器
        context.unregisterReceiver(getWidgetSyncStateReceiver(context));
    }

    /**
     * 获取小部件的笔记数据同步完成监听器
     * @param context 应用程序上下文
     * @return 小部件的笔记数据同步完成监听器
     */
    private BroadcastReceiver getWidgetSyncCompletedReceiver(Context context) {
        return new BroadcastReceiver() {
            @Override
            public void onReceive(Context context, Intent intent) {
                // 处理笔记数据同步完成
                int appWidgetId = intent.getIntExtra("app_widget_id", -1);
                Bundle extras = intent.getExtras();
                if (extras != null) {
                    List<Notes.Note> syncedNotes = extras.getParcelableArrayList("synced_notes");
                    if (syncedNotes != null) {
                        updateNotes(context, appWidgetId, syncedNotes);
                    }
                }
            }
        };
    }

    /**
     * 注册小部件的笔记数据同步完成监听器
     * @param context 应用程序上下文
     */
    private void registerWidgetSyncCompletedReceiver(Context context) {
        // 注册笔记数据同步完成监听器
        IntentFilter filter = new IntentFilter("com.example.notes.widget.action.NOTE_SYNCED");
        context.registerReceiver(getWidgetSyncCompletedReceiver(context), filter);
    }

    /**
     * 注销小部件的笔记数据同步完成监听器
     * @param context 应用程序上下文
     */
    private void unregisterWidgetSyncCompletedReceiver(Context context) {
        // 注销笔记数据同步完成监听器
        context.unregisterReceiver(getWidgetSyncCompletedReceiver(context));
    }

    /**
     * 获取小部件的笔记数据同步错误监听器
     * @param context 应用程序上下文
     * @return 小部件的笔记数据同步错误监听器
     */
    private BroadcastReceiver getWidgetSyncErrorReceiver(Context context) {
        return new BroadcastReceiver() {
            @Override
            public void onReceive(Context context, Intent intent) {
                // 处理笔记数据同步错误
                String errorMessage = intent.getStringExtra("error_message");
                Toast.makeText(context, "笔记数据同步失败: " + errorMessage, Toast.LENGTH_SHORT).show();
            }
        };
    }

    /**
     * 注册小部件的笔记数据同步错误监听器
     * @param context 应用程序上下文
     */
    private void registerWidgetSyncErrorReceiver(Context context) {
        // 注册笔记数据同步错误监听器
        IntentFilter filter = new IntentFilter("com.example.notes.widget.action.NOTE_SYNC_ERROR");
        context.registerReceiver(getWidgetSyncErrorReceiver(context), filter);
    }

    /**
     * 注销小部件的笔记数据同步错误监听器
     * @param context 应用程序上下文
     */
    private void unregisterWidgetSyncErrorReceiver(Context context) {
        // 注销笔记数据同步错误监听器
        context.unregisterReceiver(getWidgetSyncErrorReceiver(context));
    }
}